// 公共函数

// 处理校验规则
/**
 *  @Description: 业务
 *              1. 校验规则类型唯一，通过内置的正则来匹配；或者使用自定义输入的正则；增加RegExpString字段，来存储正则表达式；
 *              2. 需要处理考虑处理 number,digits,email,identity,mobile,bankAccount;[required,emphasis,sensitive]
 *              3. 需要处理时间类 timeControl = ["时间", "仅时间", "日期", "日期月份", "日期年份"]；
 *              4. 业务逻辑：正则类只有一个 number,digits,email,identity,mobile,bankAccount，timeControl，RegExp=....;
 *                 样式类可以有多个[required,emphasis,sensitive]
 *              5. 特殊情况：文本类的字段，返回的validateRule内会有maxLength和maxlength两个，这时候需要使用maxlength并且乘以2（数据库使用）
 *                 使用replace替换掉没，然后统计长度
 * */
import {RegExpObject, dateArray} from './constant';

const returnMessage = (key) => {
  let obj = {
    number: '请输入数字',
    digits: '请输入整数',
    email: '请输入符合规则的邮箱账号',
    identity: '请输入正确的身份证号',
    mobile: '请输入正确的手机号',
    bankAccount: '请输入正确的银行卡账号',
    others: '输入不符合规则，请查看'
  };
  return obj[key];
};
const handleSinglePattern = (pattern, obj) => {
  /**
   *  @Description:
   *                1. 首先判断是否有冒号，确定是否是maxlength的校验规则；
   *                2. 分别判断是不是 required、emphasis、sensitive等样式熟悉；
   *                3. 类型判断并设定正则，分别为内置的六种和按照RegExp=...规则自己输入的；
   *                4. 如果不是正则类型，则最后一种是时间类型；
   * */

  /**
   *  obj可能存在的字段如下
   *  maxLength,minLength;
   *  required,emphasis,sensitive;
   *  number,digits,email,identity,mobile,bankAccount;
   *  timeControl;
   * */
  if (pattern.indexOf(':') > -1) {
    const maxLengthArr = pattern.split(':');
    if (maxLengthArr[0] === 'maxlength') {
      obj.minLength = null;
      obj.maxLength = parseInt(maxLengthArr[1]) * 2;
    }
    if (maxLengthArr[0] === 'maxLength') {
      obj.minLength = null;
      obj.maxLength = parseInt(maxLengthArr[1]);
    }
  } else if (['required', 'emphasis', 'sensitive'].includes(pattern)) {
    obj[pattern] = true;
  } else if (pattern.indexOf('RegExp=') > -1) {
    obj.RegExp = pattern.replace('RegExp=', '');
    obj.message = returnMessage('others');
  } else if (['number', 'digits', 'email', 'identity', 'mobile', 'bankAccount'].includes(pattern)) {
    obj.RegExp = RegExpObject[pattern];
    obj.message = returnMessage(pattern);
  } else if (pattern === 'timeControl') {
    // obj.isDateArray = true; ??问题待解决
  }
};
const handlePattern = ValidateRule => {
  const obj = {};
  // 如果不存在校验规则，直接返回
  if (!ValidateRule) {
    return {};
  }
  // 如果只有一条校验规则
  if (ValidateRule.indexOf(',') < 0) {
    handleSinglePattern(ValidateRule, obj)
  } else {
    const arr = ValidateRule.split(',');
    // 如果存在maxLength并且存在maxlength(后端为了确保存储正常，将数据库该字段可存储长度也一并返回了：maxlength)
    let _index1 = arr.indexOf('maxLength');
    let _index2 = arr.indexOf('maxlength');
    if (_index1 > -1 && _index2 > -1) {
      arr.splice(_index1,1)
    }
    arr.forEach(item => {
      handleSinglePattern(item, obj)
    });
  }
  return obj;
};

export {handlePattern};
