import React, { useMemo, useState, forwardRef, useImperativeHandle } from 'react'
import { Row, DatePicker, Select, Button, Input, message, Upload, Dropdown, Menu, Form } from 'antd'
import 'dayjs/locale/zh-cn'
import locale from 'antd/es/date-picker/locale/zh_CN'
import { PlusOutlined, DeleteOutlined, ImportOutlined, ExportOutlined, QuestionCircleOutlined, EditOutlined, DownOutlined } from '@ant-design/icons'
import styles from './search.less'
import moment from 'moment'
import { ImportAccountData } from '../../../../apis/process'

const { Search } = Input;
const { RangePicker } = DatePicker;
const dateFormat = 'YYYY-MM-DD';

const SearchGroup = forwardRef((props, ref) => {

  useImperativeHandle(ref, () => ({
    getQueryInfo,
  }))

  const { accountName, config, notUse, checkName, btnsClick, keys } = props
  const { enableBatchOperation, enableImportExport, enablePrint, enableQuickSearch, enableTimeFilter, parent } = config
  const [form] = Form.useForm()
  const [dateArray, setDateArray] = useState([])
  const [formatterStr, setFormatterStr] = useState('YYYY-MM-DD HH:mm:ss')
  const [queryInfo, setQueryInfo] = useState({ info: '', timeFrom: '', timeTo: '' })

  const rangerOptions = useMemo(() => {
    const todayRanger = [
      moment(moment().format(`${dateFormat} 00:00:00`)),
      moment(moment().format(`${dateFormat} 23:59:59`)),
    ];
    const yesterdayRanger = [
      moment(
        moment()
          .subtract(1, 'days')
          .format(`${dateFormat} 00:00:00`),
      ),
      moment(
        moment()
          .subtract(1, 'days')
          .format(`${dateFormat} 23:59:59`),
      ),
    ];
    const currentWeekRanger = [
      moment(
        moment()
          .startOf('week')
          .format(`${dateFormat} 00:00:00`),
      ),
      moment(
        moment()
          .endOf('week')
          .format(`${dateFormat} 23:59:59`),
      ),
    ];
    const lastWeekRanger = [
      moment(
        moment()
          .subtract(1, 'week')
          .startOf('week')
          .format(`${dateFormat} 00:00:00`),
      ),
      moment(
        moment()
          .subtract(1, 'week')
          .endOf('week')
          .format(`${dateFormat} 23:59:59`),
      ),
    ];
    const lastMonthRanger = [
      moment(
        moment()
          .subtract(1, 'month')
          .startOf('month')
          .format(`${dateFormat} 00:00:00`),
      ),
      moment(
        moment()
          .subtract(1, 'month')
          .endOf('month')
          .format(`${dateFormat} 23:59:59`),
      ),
    ];
    const currentMonthRanger = [
      moment(
        moment()
          .startOf('month')
          .format(`${dateFormat} 00:00:00`),
      ),
      moment(
        moment()
          .endOf('month')
          .format(`${dateFormat} 23:59:59`),
      ),
    ];
    const last30DaysRanger = [
      moment(
        moment()
          .subtract(30, 'days')
          .format(`${dateFormat} 00:00:00`),
      ),
      moment(moment().format(`${dateFormat} 23:59:59`)),
    ];
    const lastOneYearRanger = [
      moment(
        moment()
          .subtract(1, 'years')
          .format(`${dateFormat} 00:00:00`),
      ),
      moment(moment().format(`${dateFormat} 23:59:59`)),
    ];
    return {
      昨天: yesterdayRanger,
      今天: todayRanger,
      本周: currentWeekRanger,
      上周: lastWeekRanger,
      本月: currentMonthRanger,
      上月: lastMonthRanger,
      近30天: last30DaysRanger,
      近一年: lastOneYearRanger,
    };
  }, [])

  const batchMenu = useMemo(() => {
    let array = []
    if (!notUse?.includes('edit')) {
      array.push({
        label: (
          <Button
            style={{ width: '100%', textAlign: 'left' }}
            size='middle'
            type='text'
            onClick={() => btnsClick({ type: '批量修改' })}
            icon={<EditOutlined />}
          >
            批量修改
          </Button>
        ),
        key: '批量修改',
      })
    }
    if (!notUse?.includes('del')) {
      array.push({
        label: (
          <Button
            style={{ width: '100%', textAlign: 'left' }}
            size='middle'
            type='text'
            onClick={() => btnsClick({ type: '批量删除' })}
            icon={<DeleteOutlined />}
          >
            批量删除
          </Button>
        ),
        key: '批量删除',
      })
    }
    return array
  }, [enableBatchOperation, notUse, keys])

  const uploadProps = {
    action: `${window.location.origin}${'/PandaWorkFlow/WorkFlow/AccountManage/ImportAccountData'}`,
    multiple: false,
    showUploadList: false,
    headers: {
      'Content-Type': 'multipart/form-data',
    },
    withCredentials: true,
    customRequest({ action, file, headers, withCredentials }) {
      const formData = new FormData();
      formData.append('file', file);
      const _data = { accountName };
      if (checkName) {
        _data.checkName = checkName;
      }
      // 默认开启检查全部字段
      _data.isCheck = true;
      ImportAccountData(
        {
          headers: {
            'Content-Type': 'multipart/form-data',
          },
        },
        { ..._data },
        formData,
      ).then(res => {
        if (res.code === 0) {
          message.success('导入成功！');
          // getAccountData();
        } else {
          message.warning(res.msg);
        }
      });
    },
  };

  const otherMenu = [
    {
      label: (
        <Upload {...uploadProps}>
          <Button style={{ width: '170px', textAlign: 'left' }} size='middle' icon={<ImportOutlined />} type='text'>
            导入
          </Button>
        </Upload>
      ),
      key: 'uploadBtn',
    },
    {
      label: (
        <Button
          style={{ width: '100%', textAlign: 'left' }}
          size='middle'
          type='text'
          onClick={() => btnsClick({ type: '导出模板' })}
          icon={<ExportOutlined />}
        >
          导出模板
        </Button>
      ),
      key: '导出模板',
    },
    {
      label: (
        <Button
          style={{ width: '100%', textAlign: 'left' }}
          size='middle'
          type='text'
          onClick={() => btnsClick({ type: '导出全字段模板' })}
          icon={<ExportOutlined />}
        >
          导出全字段模板 <QuestionCircleOutlined style={{ fontSize: 10 }} title='导出整张台账表字段' />
        </Button>
      ),
      key: '导出全字段模板',
    },
    {
      label: (
        <Button
          style={{ width: '100%', textAlign: 'left' }}
          size='middle'
          type='text'
          onClick={() => btnsClick({ type: '导出数据' })}
          icon={<ExportOutlined />}
        >
          导出数据
        </Button>
      ),
      key: '导出数据',
    },
    {
      label: (
        <Button
          style={{ width: '100%', textAlign: 'left' }}
          size='middle'
          type='text'
          onClick={() => btnsClick({ type: '导出全字段数据' })}
          icon={<ExportOutlined />}
        >
          导出全字段数据
          <QuestionCircleOutlined style={{ fontSize: 10, color: 'red' }} title='导出整张表全字段数据，数据量大时可能导致崩溃，请按需谨慎使用！' />
        </Button>
      ),
      key: '导出全字段数据',
    }
  ]

  const onSearch = e => {
    setQueryInfo({ ...queryInfo, info: e });
    getQueryInfo(form.getFieldsValue());
  }

  const onValuesChange = values => {
    let keys = Object.keys(values);
    if (keys.includes('info')) return;
    getQueryInfo(form.getFieldsValue());
  }

  const getQueryInfo = data => {
    let params = {};
    params.timeFrom = (data.time && data.time[0].format('YYYY-MM-DD HH:mm:ss')) || '';
    params.timeTo = (data.time && data.time[1].format('YYYY-MM-DD HH:mm:ss')) || '';
    params.info = data.info;
    props && props.onChange(params);
    return params;
  };

  const reset = () => {
    form.resetFields()
    btnsClick({ type: '重置' })
  }

  return (
    <Row className={styles.controlRow}>
      <Form layout={'inline'} form={form} onValuesChange={onValuesChange}>
        {
          enableTimeFilter ? (
            <Form.Item label={`录入时间`} name="time" style={{ marginBottom: 0 }}>
              <RangePicker
                size="middle"
                value={[...dateArray]}
                locale={locale}
                showTime={formatterStr === 'YYYY-MM-DD HH:mm:ss'}
                ranges={{ ...rangerOptions }}
                format={formatterStr}
                style={{ width: 300 }}
              />
            </Form.Item>
          ) : null
        }
        {
          enableQuickSearch ? (
            <Form.Item label={'快捷搜索'} name="info" style={{ marginBottom: 0 }}>
              <Search
                size="middle"
                style={{
                  width: 320,
                  verticalAlign: 'middle',
                  marginLeft: '10px',
                }}
                placeholder="请输入值"
                onSearch={onSearch}
              />
            </Form.Item>
          ) : null
        }
        {
          (enableTimeFilter || enableQuickSearch) ? (
            <Form.Item style={{ marginBottom: 0 }}>
              <Button type="primary" onClick={() => reset()}>
                重置
              </Button>
            </Form.Item>
          ) : null
        }
      </Form>
      {/* 按钮区域 */}
      <div className={styles.controlRowRight}>
        {
          parent !== '台账选择器' ? (
            <div className={styles.controlBtns}>
              {
                enableImportExport && !notUse?.includes('other') ? (
                  <Dropdown.Button
                    style={{ marginLeft: '20px' }}
                    icon={<DownOutlined />}
                    overlay={<Menu items={otherMenu} />}
                  >
                    其他操作
                  </Dropdown.Button>
                ) : null
              }
              {
                enableBatchOperation && batchMenu.length ? (
                  <Dropdown.Button
                    style={{ marginLeft: '20px' }}
                    icon={<DownOutlined />}
                    overlay={<Menu items={batchMenu} />}
                  >
                    批量操作
                  </Dropdown.Button>
                ) : null
              }
              {
                !notUse?.includes('add') ? (
                  <Button
                    type="primary"
                    size="middle"
                    onClick={() => btnsClick({ type: '添加' })}
                    icon={<PlusOutlined />}
                    style={{ marginLeft: 20 }}
                  >
                    添加
                  </Button>
                ) : null
              }
            </div>
          ) : null
        }
      </div>
    </Row>
  )
})

export default SearchGroup
