import React, { useState } from 'react'
import { Resizable } from 'react-resizable'
import './index.less'
import classnames from 'classnames'

const ResizeableTitle = (props) => {

  const { onResize, width, onClick, ...restProps } = props
  const [resizing, setResizing] = useState(false)
  const [offset, setOffset] = useState(0)

  return (
    <Resizable
      width={width + offset}
      height={0}
      handle={
        <span
          // 有偏移量显示竖线
          className={classnames(['react-resizable-handle', offset && 'active'])}
          // 拖拽层偏移
          style={{ transform: `translateX(${offset}px)` }}
          onClick={(e) => {
            // 取消冒泡，不取消貌似容易触发排序事件
            e.stopPropagation()
            e.preventDefault()
          }}
        />
      }
      onResize={(e, { size }) => {
        // 这里只更新偏移量，数据列表其实并没有伸缩
        setOffset(size.width - width)
      }}
      onResizeStart={() => setResizing(true)}
      onResizeStop={(...arg) => {
        setTimeout(() => {
          setResizing(false)
        }, 0)
        // 拖拽结束以后偏移量归零
        setOffset(0)
        // 这里是props传进来的事件，在外部是列数据中的onHeaderCell方法提供的事件，请自行研究官方提供的案例
        onResize(...arg)
      }}
      draggableOpts={{ enableUserSelectHack: false }}
    >
      <th
        onClick={(...args) => {
          if (!resizing && onClick) {
            onClick(...args)
          }
        }}
        {...restProps}
      />
    </Resizable>
  )

}

export default ResizeableTitle