import React, { useEffect, useCallback } from 'react'
import { Input, InputNumber, DatePicker } from 'antd'
import locale from 'antd/lib/date-picker/locale/zh_CN'
import moment from 'moment'
import { debounce } from '../../../../../../utils'
import { formAutomaticComputation } from '../../../../../../apis/process'

const initUserInfo = {
  fullName: '【本人姓名】',
  Phone: '【本人电话】',
  depart: { name: '【本人部门】' }
}
const loaclPaths = ['【本人姓名】', '【本人电话】', '【本人部门】']
const formAutomatic = async (parmas, callback) => {
  const { code, data } = await formAutomaticComputation(parmas)
  if (code === 0) {
    let otherRow = {}
    let datas = [...parmas.values, ...data]
    datas.forEach(v => {
      otherRow[v.fieldName] = v.fieldValue + ''
    })
    callback(otherRow)
  }
}

const formAuto = debounce(formAutomatic)

const ValueEdit = (props) => {

  const preview = sessionStorage.getItem('FormRender')
  const userInfo = preview === 'preview' ? initUserInfo : window?.globalConfig?.userInfo || initUserInfo
  const {
    onChange,
    value,
    fieldName,
    record,
    autoArray,
    otherChange,
    presetValue,
    disabled,
    placeholder,
    widget,
    format,
    min,
    max,
    formatter,
    isStoreID,
  } = props

  useEffect(() => {
    if (widget === 'TextInput') {
      let value = presetValue
      if (loaclPaths.includes(value)) {
        if (value === '【本人姓名】') {
          if (isStoreID) {
            value = userInfo.OID + ''
          } else {
            value = userInfo.fullName
          }
        } else if (value === '【本人部门】') {
          if (isStoreID) {
            value = userInfo.depart.OID + ''
          } else {
            value = userInfo.depart.name
          }
        }
        onChange({ fieldName, fieldValue: value })
      }
    }
  }, [])

  if (widget === 'DateTime') {
    const isValid = value ? moment(value)._isValid : false
    const timeOtions = format === 'dateTime' ?
      {
        showTime: {
          minuteStep: 5,
          format: 'HH:mm'
        },
        format: 'YYYY-MM-DD HH:mm'
      }
      :
      {
        picker: format,
        showTime: false
      }
    return (
      <DatePicker
        disabled={disabled}
        placeholder={disabled ? (placeholder || '') : (placeholder || '点击选择日期')}
        value={isValid ? moment(value) : null}
        showNow
        {...timeOtions}
        style={{ width: '100%' }}
        locale={locale}
        onChange={(date) => {
          if (date && date._d) {
            onChange({ fieldName, fieldValue: moment(date._d).format('YYYY-MM-DD HH:mm:ss') })
          } else {
            onChange({ fieldName, fieldValue: '' })
          }
        }}
      />
    )
  }

  if (widget === 'NumberInput') {
    return (
      <InputNumber
        min={min || Number.MIN_SAFE_INTEGER}
        max={max || Number.MAX_SAFE_INTEGER}
        step={(formatter === '${百分比}' && isStoreFormatter) ? 0.01 : 1}
        placeholder={disabled ? (placeholder || '') : (placeholder || '请输入内容')}
        disabled={disabled}
        value={value === '' ? null : Number(value)}
        onChange={(value) => {
          onChange({ fieldName, fieldValue: value !== null ? `${value}` : '' })
          if (autoArray.length) {
            autoArray.forEach(v => {
              if (v.fields.includes(fieldName)) {
                let values = v.fields.map(s => {
                  if (s === fieldName) {
                    return { fieldName, fieldValue: value !== null ? `${value}` : '' }
                  }
                  return { fieldName: s, fieldValue: record?.[s] || '' }
                })
                formAuto({ ...v, values }, otherChange)
              }
            })
          }
        }}
        style={{ width: '100%' }}
      />
    )
  }

  return (
    <Input
      disabled={disabled}
      style={{ color: loaclPaths.includes(presetValue) ? 'green' : '' }}
      onChange={(e) => onChange({ fieldName, fieldValue: e.target.value })}
      value={value}
    />
  )

}

export default ValueEdit