import React, { useMemo, useState, useRef, useLayoutEffect, useEffect } from 'react'
import { Button, message, Pagination } from 'antd'
import styles from './index.less'
import { GetAccountPageList, GetAccountConfigInfo, } from '../../../../apis/process'
import { PlusOutlined, VerticalAlignBottomOutlined } from '@ant-design/icons'
import BaseForm from './BaseForm'
import TablePack from '../../../Account/components/TablePack'
import { isObject, isJson, getNanoid } from '../../../../utils'
import Drag from '../../../components/Drag'
import SearchGroup from '../../../../core/Account/components/SearchGroup'

let formStateStr = '添加'

const getFormDataObj = (formData) => {
  let formDataObj = {}
  Object.keys(formData).map((k) => {
    if (isObject(formData[k])) {
      Object.keys(formData[k]).map((key) => {
        formDataObj[key] = formData[k][key] || ''
      })
    }
  })
  return formDataObj
}

const RelationForm = (props) => {

  const userID = window?.globalConfig?.userInfo?.OID || 1
  const codes = window?.pandaXform?.codes || { 工单编号: '', 事件编号: '', }
  const { value, onChange, schema, addons } = props
  const { disabled, '台账名称': accountName, '默认显示': defaultShow, '控制规则': controlShow, titleShow, '映射字段': mappedField, isEdit, otherSource } = schema
  const initParams = { user: userID, accountName: otherSource?.accountName, sortFields: '录入时间', direction: 'desc', total: 0, pageIndex: 1, pageSize: 100 }
  const [config, setConfig] = useState({ accountFieids: [], webShowFieldGroup: '', fieldGroup: '', formJson: {} })
  const [dataSource, setDataSource] = useState([])
  const [visible, setVisible] = useState(false)
  const [formState, setFormState] = useState('详情')
  const [row, setRow] = useState({})
  const [schemaValues, setSchemaValues] = useState([])
  const [loading, setLoading] = useState(false)
  const [visibleOther, setVisibleOther] = useState(false)
  const [configOther, setConfigOther] = useState({ accountFieids: [], webShowFieldGroup: '', fieldGroup: '', formJson: {} })
  const [loadingOther, setLoadingOther] = useState(false)
  const [dataSourceOther, setDataSourceOther] = useState([])
  const [paramsOther, setParamsOther] = useState(initParams)
  const [keys, setKeys] = useState([])
  const baseFormRef = useRef(null)

  const formDataObj = useMemo(() => {
    let formDataObj = {}
    if (addons) {
      formDataObj = getFormDataObj(addons?.formData)
    }
    return formDataObj
  }, [addons?.formData])

  const show = useMemo(() => {
    if (addons) {
      let str = controlShow
      try {
        let matchArr = str ? str.match(/\{(.+?)\}/g) : []
        if (Array.isArray(matchArr)) {
          matchArr?.forEach(v => {
            let value = v.replace(/{/g, '').replace(/}/g, '')
            let valueArr = value ? value.split('.') : []
            let fieldValue = formDataObj[valueArr[1]]
            str = str.replace(v, `'${fieldValue}'`)
          })
        }
        str = str.replace(/=/g, '==')
        str = str.replace(/and/g, '&&')
        str = str.replace(/or/, '||')
        if (str && eval(str)) {
          // addons.setSchemaByPath(addons.dataPath, { ...schema, hidden: !schema.hidden })
          return !defaultShow
        }
        return defaultShow
      } catch {
        return defaultShow
      }
    }
    return defaultShow
  }, [formDataObj, defaultShow])

  const localForm = useMemo(() => {
    const { addFieldGroup } = config
    let array = []
    let field = addFieldGroup ? addFieldGroup.split(',') : []
    const getFieldValue = (s, formDataObj, codes) => {
      return codes ? (codes[s.fromField] || formDataObj[s.fromField]) : formDataObj[s.fromField]
    }
    field.forEach(v => {
      if (Array.isArray(mappedField)) {
        mappedField.forEach(s => {
          if (s.toField === v) {
            array.push({ fieldName: s.toField, fieldValue: getFieldValue(s, formDataObj, codes) })
          }
        })
      }
    })
    return array
  }, [formDataObj, config, mappedField, codes, dataSource])

  const rowClick = async (v, r) => {
    if (addons) {
      setRow(r)
      if (v === '详情') {
        setSchemaValues(getItem(r))
        setVisible(true)
      } else if (v === '编辑') {
        setSchemaValues(getItem(r))
        setVisible(true)
      } else if (v === '删除') {
        deleteTableInfo(r)
      }
      formStateStr = v
      setFormState(v)
    }
  }

  const getItem = (data, max) => {
    if (Array.isArray(data)) {
      let obj = { ...row, ID: row.ID }
      data.forEach(v => {
        obj[v.fieldName] = v.fieldValue
      })
      return obj
    } else {
      let values = []
      for (let key in data) {
        if (max) {
          values.push({ FieldName: key, FieldValue: data[key] })
        } else {
          values.push({ fieldName: key, fieldValue: data[key] })
        }
      }
      return values
    }
  }

  const deleteTableInfo = (r) => {
    let listItem = { type: 'delete', accountTable: accountName, ID: r?.ID, values: getItem(r) }
    let relationForm = addons.getValue('relationForm') || { configs: [], data: [] }
    if (r?.ID?.includes('前端ID')) {
      let list = relationForm.data.filter(v => v.ID !== r.ID)
      addons.setValue('relationForm', { ...relationForm, data: list })
    } else {
      addons.setValue('relationForm', { ...relationForm, data: [...relationForm.data, listItem] })
    }
    let tableData = dataSource.filter(v => v.ID !== r.ID)
    setDataSource(tableData)
    onChange(`${tableData.length}`)
  }

  const getConfig = async () => {
    let { code, data, msg } = await GetAccountConfigInfo(accountName)
    if (code === 0) {
      setConfig({ ...data, formJson: isJson(data.formJson) ? JSON.parse(data.formJson) : {} })
      const queryWheres = getQueryWheres()
      getTableData(queryWheres)
    } else {
      message.error(msg)
    }
  }

  const getQueryWheres = () => {
    let queryWheres = []
    if (Array.isArray(mappedField)) {
      let formDataObj = getFormDataObj(addons?.formData)
      mappedField.forEach(v => {
        queryWheres.push({ field: v.toField, type: '等于', value: codes[v.fromField] || formDataObj[v.fromField] })
      })
    }
    let relationForm = addons.getValue('relationForm') || { configs: [], data: [] }
    addons.setValueByPath('relationForm',
      {
        ...relationForm,
        configs: [...relationForm.configs, { ...schema, queryWheres: queryWheres.map(v => ({ fieldName: v.field, fieldValue: v.value })) }]
      })
    return queryWheres
  }

  const getTableData = async (queryWheres) => {
    if (!addons) return
    let params = {
      user: userID,
      accountName: accountName,
      direction: 'desc',
      timeField: '录入时间',
      pageIndex: 0,
      pageSize: 0,
      queryWheres
    }
    if (queryWheres.every(v => !v.value)) return
    const { code, data, msg } = await GetAccountPageList(params)
    if (code === 0) {
      let tableData = data?.jsonData ? JSON.parse(data?.jsonData) : []
      if (Array.isArray(tableData)) {
        setDataSource(tableData)
        onChange(`${tableData.length}`)
      } else {
        setDataSource([])
        onChange(`${0}`)
      }
    } else {
      message.error(msg)
    }
  }

  const getData = () => {
    getConfig()
  }

  const onOk = async () => {
    if (addons) {
      const { formValue, errors } = await baseFormRef.current.getValues()
      if (errors.length) {
        return message.error('请按照提示完善表单内容')
      }
      let type = row.ID.includes('前端ID') ? 'add' : 'edit'
      let itemData = { type, accountTable: accountName, ID: row.ID, values: formValue }
      if (formStateStr !== '添加') {
        let array = []
        if (Array.isArray(dataSource)) {
          dataSource.forEach(v => {
            if (v.ID === row.ID) {
              array.push({ ...v, ...getItem(formValue) })
            } else {
              array.push({ ...v })
            }
          })
        }
        setDataSource(array)
        setTimeout(() => {
          onChange(value && value.includes(' ') ? `${array.length}` : `${array.length} `)
        }, 0)
      } else if (formStateStr === '添加') {
        let array = [getItem(formValue), ...dataSource]
        setDataSource(array)
        setTimeout(() => {
          onChange(`${array.length}`)
        }, 0)
      }
      let relationForm = addons.getValue('relationForm') || { configs: [], data: [] }
      let listArray = relationForm.data.filter(v => v.ID !== itemData.ID)
      listArray.push({ ...itemData })
      addons.setValue('relationForm', { ...relationForm, data: listArray })
      setVisible(false)
    }
  }

  const addClick = () => {
    if (addons) {
      if (!localForm.length) {
        return message.info('台账添加字段未配置!')
      }
      setRow({ ID: `前端ID${new Date().getTime()}` })
      setFormState('添加')
      setSchemaValues(localForm)
      formStateStr = '添加'
      setVisible(true)
    }
  }

  const btnsClick = ({ type, row }) => {
    if (type === '变化') {
      let array = []
      dataSource.forEach(v => {
        if (v.ID === row.ID) {
          array.push({ ...row })
        } else {
          array.push({ ...v })
        }
      })
      setDataSource(array)
      let type = row?.ID?.includes('前端ID') ? 'add' : 'edit'
      let formValue = []
      let field = config?.accountFieids?.filter(v => v.webDisplay).map(v => v.fieldName)
      field.forEach(v => {
        formValue.push({ fieldName: v, fieldValue: row[v] || '' })
      })
      let itemData = { type, accountTable: accountName, ID: row.ID, values: formValue }
      let relationForm = addons?.getValue('relationForm') || { configs: [], data: [] }
      let listArray = relationForm.data.filter(v => v.ID !== itemData.ID)
      listArray.push(itemData)
      addons?.setValue('relationForm', { ...relationForm, data: listArray })
      onChange(`${new Date().getTime()}`)
      addons?.setValue(addons?.dataPath, `${array.length}`)
    } else {
      rowClick(type, row)
    }
  }

  const getFieldValue = (s, formDataObj, codes) => {
    return codes ? (codes[s.fromField] || formDataObj[s.fromField]) : formDataObj[s.fromField]
  }

  const addRow = () => {
    if (addons) {
      let array = []
      let field = config?.accountFieids?.filter(v => v.webDisplay).map(v => v.fieldName)
      let obj = {}
      field.forEach(v => {
        obj[v] = ''
        if (Array.isArray(mappedField)) {
          mappedField.forEach(s => {
            if (s.toField === v) {
              let value = getFieldValue(s, formDataObj, codes) || ''
              array.push({ fieldName: s.toField, fieldValue: value })
              obj[s.toField] = value
            }
          })
        }
      })
      let ID = `${getNanoid(10, '前端ID')}`
      let item = {
        type: 'add',
        accountTable: accountName,
        ID: ID,
        values: array
      }
      setDataSource([...dataSource, { ID: ID, ...obj }])
      let relationForm = addons.getValue('relationForm') || { configs: [], data: [] }
      relationForm.data.push(item)
      addons.setValue('relationForm', { ...relationForm })
      setTimeout(() => {
        onChange(value && value.includes(' ') ? `${array.length}` : `${array.length} `)
      }, 0)
    }
  }

  const addRows = () => {
    getConfigOther()
    setVisibleOther(true)
  }

  const getConfigOther = async () => {
    let { code, data, msg } = await GetAccountConfigInfo(otherSource.accountName)
    if (code === 0) {
      setConfigOther({ ...data, formJson: isJson(data.formJson) ? JSON.parse(data.formJson) : {} })
      getDataSourceOther({})
    } else {
      message.error(msg)
    }
  }

  const getDataSourceOther = async (values) => {
    setLoadingOther(true)
    const { code, data } = await GetAccountPageList({ ...paramsOther, ...values })
    if (code === 0) {
      setDataSourceOther(data?.jsonData ? JSON.parse(data?.jsonData) : [])
      setParamsOther({ ...paramsOther, total: data.totalCount })
    }
    setLoadingOther(false)
  }

  const setDataSources = (array) => {
    let i = 0
    let timer = setInterval(() => {
      if (i < array.length) {
        setDataSource(dataSource => [...dataSource, array[i]])
        i++
      } else {
        clearInterval(timer)
      }
    }, 100)
  }

  const rowsOk = () => {
    if (!keys.length) {
      return message.info('请选择行数据！')
    }
    const { accountFieids } = config
    let array = []
    let items = []
    let field = accountFieids.filter(v => v.webDisplay).map(v => v.fieldName)
    dataSourceOther.forEach(v => {
      if (keys.includes(v.ID)) {
        let ID = `${getNanoid(10, '前端ID')}`
        let values = []
        let obj = { ID: ID }
        field.forEach(s => {
          if (mappedField.some(v => v.toField === s)) {
            if (Array.isArray(mappedField)) {
              mappedField.forEach(j => {
                if (j.toField === s) {
                  let value = getFieldValue(j, formDataObj, codes) || ''
                  values.push({ fieldName: s, fieldValue: value })
                  obj[s] = value
                }
              })
            }
          } else {
            let fromField = otherSource?.field?.find(j => j.toField === s)?.fromField
            obj[s] = v[fromField] || ''
            values.push({ fieldName: s, fieldValue: v[fromField] || '' })
          }
        })
        let item = {
          type: 'add',
          accountTable: accountName,
          ID: ID,
          values: values
        }
        items.push(item)
        array.push(obj)
      }
    })
    setDataSources(array)
    // setDataSource([...dataSource, ...array])
    setKeys([])
    let relationForm = addons.getValue('relationForm') || { configs: [], data: [] }
    let listArray = relationForm.data.concat(items)
    addons.setValue('relationForm', { ...relationForm, data: listArray })
    setTimeout(() => {
      onChange(value && value.includes(' ') ? `${array.length}` : `${array.length} `)
    }, 0)
    setVisibleOther(false)
  }

  const search = (values) => {
    setParamsOther({ ...paramsOther, ...values })
    getDataSourceOther({ ...values })
  }

  const pageChange = (pageIndex, pageSize) => {
    getDataSourceOther({ pageIndex, pageSize })
  }

  useEffect(() => {
    //初始化关联表单配置信息，初始显示隐藏
    if (addons) {
      getData()
      // console.log('addons', addons)
      // setTimeout(() => {
      //   let queryWheres = []
      //   if (Array.isArray(mappedField)) {
      //     let formDataObj = getFormDataObj(addons?.formData)
      //     mappedField.forEach(v => {
      //       queryWheres.push({ field: v.toField, type: '等于', value: codes[v.fromField] || formDataObj[v.fromField] })
      //     })
      //   }
      //   let relationForm = addons.getValue('relationForm') || { configs: [], data: [] }
      //   addons.setValueByPath('relationForm', { ...relationForm, configs: [...relationForm.configs, { ...schema, queryWheres }] })

      // }, 0)
    }
  }, [])

  return (
    <div className={styles.relationForm} style={{ display: (addons && !show) ? 'none' : 'block' }}>
      <div>
        <div className={styles.header}>
          <div className={styles.text}>{titleShow}</div>
          {
            !disabled ? (
              <div className={styles.btns}>
                <Button style={{ marginRight: '5px' }} size='middle' icon={<PlusOutlined />} onClick={isEdit ? addRow : addClick}>{isEdit ? '添加一行' : '添加'}</Button>
                {otherSource?.accountName ? <Button type='primary' size='middle' icon={<VerticalAlignBottomOutlined />} onClick={addRows}>其他台账导入</Button> : null}
              </div>
            ) : null
          }
        </div>
        <TablePack
          loading={loading}
          setDataSource={setDataSource}
          notUse={disabled ? 'edit,add,del' : ''}
          config={{ ...config, parent: '关联表单' }}
          dataSource={dataSource}
          btnsClick={btnsClick}
          isEdit={isEdit}
          otherSource={otherSource}
        />
      </div>
      <Drag
        title={accountName}
        width='80%'
        bodyStyle={{ height: '700px', overflow: 'auto' }}
        visible={visible}
        onOk={onOk}
        onCancel={() => setVisible(false)}
        destroyOnClose
      >
        <BaseForm
          schemaForm={{
            group: config.formJson,
            values: schemaValues,
          }}
          formState={formState}
          config={config}
          ref={baseFormRef}
          disabled={!['编辑', '添加'].includes(formState)}
        />
      </Drag>
      <Drag
        visible={visibleOther}
        title={otherSource?.accountName}
        width='60%'
        onCancel={() => setVisibleOther(false)}
        onOk={() => rowsOk()}
        bodyStyle={{ height: 570, paddingBottom: 0 }}
        getContainer={false}
      >
        <div className={styles.content}>
          <div className={styles.top}>
            <SearchGroup
              onChange={search}
              notUse={'edit,add,del'}
              btnsClick={btnsClick}
              config={{ ...config, parent: '台账选择器' }}
            />
          </div>
          <div className={styles.bottom}>
            <TablePack
              loading={loadingOther}
              notUse={disabled ? 'edit,add,del' : ''}
              config={{ ...configOther, parent: '台账选择器' }}
              dataSource={dataSourceOther}
              rowSelection={{
                type: 'checkbox',
                selectedRowKeys: keys,
                fixed: 'left',
                onChange: (keys) => setKeys(keys)
              }}
            />
          </div>
          <div className={styles.footer}>
            <div className={styles.total}>共计{paramsOther.total}条数据</div>
            <div className={styles.pagination}>
              <Pagination
                size='small'
                showQuickJumper
                pageSize={paramsOther.pageSize}
                current={paramsOther.pageIndex}
                total={paramsOther.total}
                onChange={pageChange}
              />
            </div>
          </div>
        </div>
      </Drag>
    </div>
  )

}

export default RelationForm