import React, { useEffect, useState } from 'react'
import { TreeSelect, message } from 'antd'
import { DownOutlined } from '@ant-design/icons'
import { getDeptList } from '../../../../apis/process'

const getList = (deptList) => {
  if (Array.isArray(deptList)) {
    deptList.forEach(v => {
      v.deptId = v.deptId + ''
      getList(v.childDepts)
    })
  }
  return deptList
}

const DeptSelector = (props) => {

  const { value, onChange, schema, addons } = props
  const { isMultiple, disabled, placeholder, presetValue, isStoreID } = schema
  const property = isStoreID ? 'deptId' : 'deptName'
  const [treeData, setTreeData] = useState([])

  const treeChange = (value) => {
    if (addons) {
      addons.setValue(addons.dataPath, isMultiple ? value.join(',') : value)
    }
  }

  const filterTreeNode = (inputValue, treeNode) => {
    if (treeNode.deptName.includes(inputValue)) {
      return true
    }
    return false
  }

  const iconClick = async () => {
    if (addons) {
      const { code, data, msg } = await getDeptList()
      if (code === 0) {
        setTreeData(getList(data))
      } else {
        message.error(msg)
      }
    }
  }

  useEffect(() => {
    if (addons) {
      addons.setValue(addons.dataPath, presetValue)
    } else {
      onChange(presetValue)
    }
  }, [presetValue])

  useEffect(() => {
    if (isStoreID) {
      iconClick()
    }
  }, [])

  return (
    <TreeSelect
      style={{ width: '100%' }}
      disabled={disabled}
      multiple={isMultiple}
      switcherIcon={<DownOutlined />}
      fieldNames={{ label: 'deptName', value: property, children: 'childDepts' }}
      treeData={treeData}
      onChange={treeChange}
      onFocus={iconClick}
      value={isMultiple ? (value ? value.split(',') : []) : value}
      filterTreeNode={filterTreeNode}
      showSearch
      showArrow={!disabled}
      allowClear
      treeDefaultExpandAll={true}
      placeholder={disabled ? (placeholder || '') : (placeholder || '点击选择部门')}
    >
    </TreeSelect>
  )

}

export default DeptSelector