import React, { useState, useRef, useEffect, useContext } from 'react'
import { Input, Button } from 'antd'
import { CompassOutlined, DownOutlined } from '@ant-design/icons'
import {
  ArcGISSceneMap,
  ClickQuery,
  geomUtils,
  GraphicsLayer,
  Graphic
} from '@wisdom-map/arcgismap'
import point from './img/point.png'
import { isObject } from '../../../../utils'
import Drag from '../../../components/Drag'
import styles from './index.less'

let geometrystr = ''
let gisInfo = {}
let layer = 0

const Device = (props) => {

  const { token, client } = window.globalConfig
  const { addons, value, onChange, schema } = props
  const { placeholder, disabled, parent, presetValue, screenShot, fieldshine } = schema
  const [visible, setVisible] = useState(false)
  const [text, setText] = useState(presetValue ? presetValue : '（空）')
  const [put, setPut] = useState(true)
  let clickref = useRef(null)
  let view = useRef(null)

  const getView = (mapObj, Map) => {
    setTimeout(() => {
      if (mapObj) {
        view.current = mapObj
        let arr = (value && value.split(',')) || []
        let obj = (arr.length === 2) ? { point: { x: Number(arr[0]), y: Number(arr[1]) } } : {}
        if (arr.length === 2) {
          view.current.goTo({
            center: geomUtils.toGeometry({ x: Number(arr[0]), y: Number(arr[1]) }),
            zoom: 15
          })
        }
        clickref.current = new ClickQuery({
          view: view.current,
          offsetX: -150,
          offsetY: -310,
          continuous: true,
          onlyWhatch: disabled,
          ...obj,
          callback: ({ clickPoint, matchPoint, data }) => {
            gisInfo = {}
            if (layer) {
              view.current.map.remove(layer)
            }
            if (matchPoint) {
              geometrystr = `${matchPoint.x},${matchPoint.y}`
              gisInfo = { layerName: data.layerName, gisCode: data.attributes.code }
            } else if (clickPoint) {
              geometrystr = `${clickPoint.x},${clickPoint.y}`
              layer = new GraphicsLayer({
                id: 'ok-layers',
              })
              if (geometrystr) {
                const polypointGraphic = new Graphic({
                  geometry: geomUtils.toGeometry({
                    x: Number(geometrystr.split(',')[0]),
                    y: Number(geometrystr.split(',')[1])
                  }),
                  symbol: {
                    type: "picture-marker",
                    url: point,
                    width: "20px",
                    height: "20px"
                  },
                })
                layer.add(polypointGraphic)
                view.current.map.add(layer)
              }
            }
          }
        })
      }
    }, 500)
  }

  const getMapInfo = (mapObj, Map) => {
    setTimeout(() => {
      if (mapObj) {
        view.current = mapObj
        let arr = (value && value.split(',')) || []
        let obj = (arr.length === 2) ? { point: { x: Number(arr[0]), y: Number(arr[1]) } } : {}
        if (arr.length === 2) {
          view.current.goTo({
            center: geomUtils.toGeometry({ x: Number(arr[0]), y: Number(arr[1]) }),
            zoom: 15
          })
        }
        clickref.current = new ClickQuery({
          view: view.current,
          offsetX: -150,
          offsetY: -310,
          continuous: true,
          onlyWhatch: true,
          ...obj,
          callback: ({ clickPoint, matchPoint, data }) => {
            gisInfo = {}
            if (layer) {
              view.current.map.remove(layer)
            }
            if (matchPoint) {
              geometrystr = `${matchPoint.x},${matchPoint.y}`
              gisInfo = { layerName: data.layerName, gisCode: data.attributes.code }
            } else if (clickPoint) {
              geometrystr = `${clickPoint.x},${clickPoint.y}`
              layer = new GraphicsLayer({
                id: 'ok-layers',
              })
              if (geometrystr) {
                const polypointGraphic = new Graphic({
                  geometry: geomUtils.toGeometry({
                    x: Number(geometrystr.split(',')[0]),
                    y: Number(geometrystr.split(',')[1])
                  }),
                  symbol: {
                    type: "picture-marker",
                    url: point,
                    width: "20px",
                    height: "20px"
                  },
                })
                layer.add(polypointGraphic)
                view.current.map.add(layer)
              }
            }
          }
        })
      }
    }, 500)
  }

  const iconClick = () => {
    if (addons) {
      setVisible(true)
    }
  }

  const onOk = () => {
    onChange(geometrystr)
    let { paths } = addons.getSchemaByPath('#')
    if (Array.isArray(paths)) {
      paths.forEach(v => {
        let values = addons.getValue(v)
        if (isObject(values)) {
          for (let key in values) {
            if (key === 'GIS编码' && gisInfo.gisCode) {
              addons.setValue(`${v}.${key}`, gisInfo.gisCode)
            }
            if (key === 'GIS图层' && gisInfo.layerName) {
              addons.setValue(`${v}.${key}`, gisInfo.layerName)
            }
          }
        }
      })
    }
    setVisible(false)
  }

  const onCancel = () => {
    setVisible(false)
  }

  useEffect(() => {
    if (addons) {
      if (presetValue) {
        geometrystr = presetValue
      }
      addons.setValueByPath(addons.dataPath, presetValue)
    } else {
      onChange(presetValue)
    }
  }, [presetValue])

  useEffect(() => {
    if (disabled && screenShot) {
      addons?.setSchemaByPath(addons.dataPath, { ...schema, width: '67%' })
    }
  }, [disabled, screenShot])

  if (screenShot && disabled && addons) {
    return (
      <div className={styles.deviceShot}>
        <div className={styles.content}>
          <div className={styles.top}>
            <div className={styles.text}>{text}</div>
            <div
              className={styles.icon}
              onClick={() => setPut(!put)}
              style={{ transform: `rotate(${put ? 0 : 180}deg)` }}
            >
              <DownOutlined />
            </div>
          </div>
          <div className={styles.mapBox} style={{ height: put ? '208px' : '0px' }}>
            <ArcGISSceneMap
              getMapInfo={getMapInfo}
              widgets={[]}
              token={token}
              client={token ? client : 'sandbox'}
            />
          </div>
        </div>
      </div>
    )
  }

  return (
    <div>
      {
        !disabled ?
          <Input
            disabled={disabled}
            value={value}
            placeholder={disabled ? (placeholder || '') : (placeholder || '点击选择设备')}
            onClick={iconClick}
            addonAfter={disabled ? null : <CompassOutlined style={{ color: 'rgba(0, 0, 0, 0.25)' }} onClick={iconClick} />}
            style={{ width: '100%' }}
          /> :
          <Button
            icon={<CompassOutlined style={{ color: value ? '#0092fe' : 'rgba(0, 0, 0, 0.25)' }} />}
            onClick={() => {
              if (value) {
                iconClick()
              }
            }}
          >
            {value ? '查看设备' : '无设备信息'}
          </Button>
      }
      <Drag
        visible={visible}
        title='设备选择'
        width={'80%'}
        onOk={onOk}
        onCancel={onCancel}
        cancelText={'取消'}
        okText={'确定'}
        bodyStyle={{ height: 600, overflowY: 'auto', position: 'relative' }}
        {...disabled ? { footer: null } : {}}
      >
        <ArcGISSceneMap
          getMapInfo={getView}
          widgets={[]}
          token={token}
          client={token ? client : 'sandbox'}
        />
      </Drag>
    </div >
  )
}

export default Device