import React, { useState, useEffect, useRef } from 'react'
import { Input, Button } from 'antd'
import { CompassOutlined, DownOutlined } from '@ant-design/icons'
import styles from './index.less'
import {
  ArcGISSceneMap,
  AutoCompleteSearch,
  Drawtool as drawTool,
  Graphic,
  GraphicsLayer,
  Point,
  geomUtils,
  SimpleLineSymbol,
} from '@wisdom-map/arcgismap'
import Drag from '../../../components/Drag'
import { isJson, isObject, mercatorToLngLat } from '../../../../utils'
import { getLocation } from '../../../../apis/process'

const DrawPath = (props) => {

  const { token, client } = window.globalConfig
  const { value, onChange, schema, addons } = props
  const { disabled, placeholder, presetValue, addressSync, screenShot } = schema
  const [visible, setVisible] = useState(false)
  const [paths, setPaths] = useState({})
  const [text, setText] = useState(presetValue ? '已绘制' : '（空）')
  const [put, setPut] = useState(true)
  const viewRef = useRef(null)
  const layers = useRef(null)

  const createPaths = (view) => {
    setTimeout(() => {
      if (isJson(value)) {
        let layer = new Graphic({
          geometry: geomUtils.toGeometry(JSON.parse(value)),
          symbol: new SimpleLineSymbol({

          }),
        })
        layers.current = new GraphicsLayer()
        layers.current.add(layer)
        view.map.add(layers.current)
      }
    }, 2000)
  }

  const getView = (viewObject) => {
    setPaths(isJson(value) ? JSON.parse(value) : '')
    viewRef.current = viewObject
    createPaths(viewObject)
  }

  const getMapInfo = (viewObject) => {
    setPaths(isJson(value) ? JSON.parse(value) : '')
    viewRef.current = viewObject
    createPaths(viewObject)
  }

  const getPoints = (view) => {
    if (view) {
      view.map.remove(layers.current)
      drawTool.activate({
        view,
        action: 'polyline',
        target: 'erroSendUpWidget',// 随便写
        toolTip: '左键选择位置',
        drawEnd: geometry => {
          setPaths({ paths: geometry.paths })
          let layer = new Graphic({
            geometry: geometry,
            symbol: new SimpleLineSymbol({

            }),
          })
          layers.current = new GraphicsLayer()
          layers.current.add(layer)
          view.map.add(layers.current)
        },
        // rightClick: () => { getPoints(view, coordGetLayer) }
      })
    }
  }

  const start = () => {
    getPoints(viewRef.current)
  }

  const showMap = () => {
    setVisible(true)
  }

  const onCancel = () => {
    setVisible(false)
  }

  const onOk = () => {
    onChange(JSON.stringify(paths))
    setVisible(false)
  }

  useEffect(() => {
    if (addons) {
      addons.setValueByPath(addons.dataPath, presetValue)
    } else {
      onChange(presetValue)
    }
  }, [presetValue])

  useEffect(() => {
    if (disabled && screenShot) {
      addons?.setSchemaByPath(addons.dataPath, { ...schema, width: '67%' })
    }
  }, [disabled, screenShot])

  if (screenShot && disabled && addons) {
    return (
      <div className={styles.drawPathShot}>
        <div className={styles.content}>
          <div className={styles.top}>
            <div className={styles.text}>{text}</div>
            <div
              className={styles.icon}
              onClick={() => setPut(!put)}
              style={{ transform: `rotate(${put ? 0 : 180}deg)` }}
            >
              <DownOutlined />
            </div>
          </div>
          <div className={styles.mapBox} style={{ height: put ? '208px' : '0px' }}>
            <ArcGISSceneMap
              getMapInfo={getMapInfo}
              widgets={[]}
              token={token}
              client={token ? client : 'sandbox'}
            />
          </div>
        </div>
      </div>
    )
  }

  return (
    <div>
      {
        !disabled ?
          <Input
            disabled={disabled}
            placeholder={disabled ? (placeholder || '') : (placeholder || '点击绘制路径')}
            value={isJson(value) ? '已绘制' : ''}
            addonAfter={disabled ? null : <CompassOutlined style={{ color: 'rgba(0, 0, 0, 0.25)' }} onClick={showMap} />}
            style={{ width: '100%' }}
            onClick={showMap}
          />
          :
          <Button
            icon={<CompassOutlined style={{ color: value ? '#0092fe' : 'rgba(0, 0, 0, 0.25)' }} />}
            onClick={() => {
              if (value) {
                showMap()
              }
            }}
          >
            {value ? '查看路径' : '无路径信息'}
          </Button>
      }
      <Drag
        width={'80%'}
        title="选取坐标"
        visible={visible}
        onOk={onOk}
        onCancel={onCancel}
        cancelText={'取消'}
        okText={'确定'}
        bodyStyle={{ height: 600, overflowY: "auto", position: 'relative' }}
        destroyOnClose={true}
        {...disabled ? { footer: null } : {}}
      >
        <div style={{ height: '90%' }}>
          <Button
            style={{ position: 'absolute', zIndex: '99', right: '10px', top: '10px' }}
            type='primary'
            onClick={start}
          >
            开始绘制
          </Button>
          <ArcGISSceneMap
            getMapInfo={getView}
            widgets={[]}
            token={token}
            client={token ? client : 'sandbox'}
          />
        </div>
      </Drag>
    </div >
  )

}

export default DrawPath