// 自定义时间组件
import React, { useEffect } from 'react'
import { DatePicker } from 'antd'
import moment from 'moment'
import locale from 'antd/lib/date-picker/locale/zh_CN'

const DateTime = (props) => {

  const { value, onChange, schema, addons } = props
  const { disabled, format, presetValue, placeholder, options, currentDate, defaultCurrent } = schema

  useEffect(() => {
    if (addons) {
      let value = presetValue || (options === '默认为当前时间' ? moment(new Date()).format('YYYY-MM-DD HH:mm:ss') : '')
      addons.setValue(addons.dataPath, value)
    } else {
      onChange(presetValue)
    }
  }, [presetValue, options])

  const disabledDate = (current) => {
    if (options === '不超过当前时间') {
      return current && current > moment()
    } else {
      return false
    }
  }

  const dateChange = (date, dateStr) => {
    if (date && date._d) {
      onChange(moment(date._d).format('YYYY-MM-DD HH:mm:ss'))
    } else {
      onChange('')
    }
  }

  const isValid = moment(value)._isValid
  const timeOtions = format === 'dateTime' ?
    {
      showTime: {
        minuteStep: 5,
        format: 'HH:mm'
      },
      format: 'YYYY-MM-DD HH:mm'
    }
    :
    {
      picker: format,
      showTime: false
    }

  return (
    <DatePicker
      disabledDate={disabledDate}
      disabled={disabled}
      placeholder={disabled ? (placeholder || '') : (placeholder || '点击选择日期')}
      value={isValid ? moment(value) : null}
      onChange={dateChange}
      showNow
      {...timeOtions}
      style={{ width: '100%' }}
      locale={locale}
    />
  );

};

export default DateTime
