import React, { useEffect } from 'react'
import { TimePicker } from 'antd'
import locale from 'antd/lib/date-picker/locale/zh_CN'
import moment from 'moment'

const Time = (props) => {

  const { value, onChange, schema, addons } = props
  const { presetValue, currentDate, defaultCurrent, options, disabled, placeholder } = schema

  const disabledDate = (current) => {
    if (options === '不超过当前时间') {
      return current && current.format('HH:mm:ss') > moment().format('HH:mm:ss')
    } else {
      return false
    }
  }

  const timeChange = (time, timeStr) => {
    if (time) {
      onChange(moment(time).format('YYYY-MM-DD HH:mm:ss'))
    } else {
      onChange('')
    }
  }

  useEffect(() => {
    if (addons) {
      let value = presetValue || (options === '默认为当前时间' ? moment().format('YYYY-MM-DD HH:mm:ss') : '')
      addons.setValue(addons.dataPath, value)
    } else {
      let value = presetValue || (options === '默认为当前时间' ? moment().format('YYYY-MM-DD HH:mm:ss') : '')
      onChange(value)
    }
  }, [presetValue, options])

  const isValid = moment(value)._isValid

  return (
    <TimePicker
      style={{ width: '100%' }}
      value={isValid ? moment(value) : null}
      onChange={timeChange}
      disabledDate={disabledDate}
      locale={locale}
      disabled={disabled}
      placeholder={disabled ? (placeholder || '') : (placeholder || '点击选择时间')}
      // getPopupContainer={(targterNode) => targterNode.parentElement || document.body}
      minuteStep={5}
      format={'HH:mm'}
    />
  )

}

export default Time