import React, { useEffect, useMemo } from 'react'
import { Cascader } from 'antd'
import city from '../../../../constant/city'

const CascadeSelector = (props) => {

  const { value, onChange, schema, addons } = props
  const { presetValue, disabled, placeholder, sourceType, } = schema

  const enums = useMemo(() => {
    if (sourceType === '城市数据') {
      return city
    }
    return []
  }, [sourceType])

  const cascaderChange = (selectedValues) => {
    if (addons) {
      onChange(selectedValues.join('/'))
    }
  }

  useEffect(() => {
    if (addons) {
      addons.setValue(addons.dataPath, presetValue)
    } else {
      onChange(presetValue)
    }
  }, [presetValue])

  return (
    <Cascader
      allowClear
      placeholder={disabled ? (placeholder || '') : (placeholder || '请选择内容')}
      fieldNames={{ label: 'label', value: 'label', children: 'children' }}
      value={value ? value.split('/') : []}
      onChange={cascaderChange}
      disabled={disabled}
      style={{ width: '100%' }}
      options={enums}
    />
  )

}

export default CascadeSelector