import React, { useState, useEffect, useMemo } from 'react'
import { Checkbox } from 'antd'
import { GetSelectItemList, GetFieldValueFromTable, getStationListByUserID, getStationIDListByUserID } from '../../../../apis/process'
import TagPack from '../../../components/TagPack'

const CheckBox = (props) => {

  const { value, onChange, schema, addons } = props
  const { presetValue, disabled, sourceType, options, dictionary, tableName, fieldName, isStoreID, color } = schema
  const [dictionaryList, setDictionaryList] = useState([])
  const [tableData, setTableData] = useState([])

  const enums = useMemo(() => {
    switch (sourceType) {
      case '手动输入':
        return Array.isArray(options) ? options : []
      case '数据字典':
        return Array.isArray(dictionaryList) ? dictionaryList.map(v => v.nodeName) : []
      case '表数据':
        return Array.isArray(tableData) ? tableData : []
      default:
        return []
    }
  }, [sourceType, options, dictionaryList, tableData])

  const children = useMemo(() => {
    let children = []
    if (Array.isArray(enums)) {
      enums.forEach(v => {
        if (sourceType === '站点') {
          children.push(<Checkbox key={isStoreID ? v.stationID : v.stationName} value={isStoreID ? v.stationID : v.stationName}>{v.stationName}</Checkbox>)
        } else if (sourceType === '手动输入') {
          children.push(
            <Checkbox key={v.value} value={v.value}>
              {
                color ? <TagPack color={v.color} text={v.value} /> : v.value
              }
            </Checkbox>
          )
        } else {
          children.push(<Checkbox key={v} value={v}>{v}</Checkbox>)
        }
      })
    }
    return children
  }, [sourceType, enums, color])

  useEffect(() => {
    if (addons) {
      addons.setValue(addons.dataPath, presetValue || '')
    } else {
      onChange(presetValue || '')
    }
  }, [presetValue])

  const onFocus = async () => {
    if (addons) {
      if (sourceType === '数据字典') {
        if (!dictionary) {
          return message.error('请选择数据字典！')
        }
        const { data } = await GetSelectItemList({ nodeName: dictionary })
        if (Array.isArray(data)) {
          setDictionaryList(data)
        }
      }
      if (sourceType === '表数据') {
        if (!tableName) {
          return message.info('请选择表名!')
        }
        if (!fieldName) {
          return message.info('请选择字段名!')
        }
        const { code, data, msg } = await GetFieldValueFromTable(tableName, fieldName)
        if (code === 0) {
          if (Array.isArray(data)) {
            setTableData(data.filter(v => v))
          } else {
            setTableData([])
          }
        } else {
          message.error(msg)
        }
      }
    }
  }

  const checkChange = (value) => {
    if (addons) {
      addons.setValue(addons.dataPath, value.join(','))
    }
  }

  useEffect(() => {
    if (sourceType === '数据字典' && dictionary) {
      onFocus()
    }
    if (sourceType === '表数据' && tableName && fieldName) {
      onFocus()
    }
  }, [sourceType, dictionary, tableName, fieldName])

  return (
    <Checkbox.Group
      disabled={disabled}
      onChange={checkChange}
      value={value ? value.split(',') : []}
    >
      {children}
    </Checkbox.Group>
  )
}

export default CheckBox