import React, { useState, useMemo, useEffect } from 'react'
import { Select, message, Tag } from 'antd'
import { GetSelectItemList, GetFieldValueFromTable, getStationListByUserID, getStationIDListByUserID } from '../../../../apis/process'
import TagPack from '../../../components/TagPack'

const { Option } = Select

const ComboBox = (props) => {

  const userID = window?.globalConfig?.userInfo?.OID || 1
  const { value, onChange, schema, addons } = props
  const { placeholder, disabled, sourceType, options, dictionary, tableName, fieldName, presetValue, isMultiple, isEdit, isMySite, isSearch, isStoreID, color } = schema
  const [dictionaryList, setDictionaryList] = useState([])
  const [tableData, setTableData] = useState([])
  const [site, setSite] = useState([])

  useEffect(() => {
    if (addons) {
      addons.setValue(addons.dataPath, presetValue || '')
      if (sourceType === '站点') {
        getStation(presetValue)
      }
    } else {
      onChange(presetValue || '')
    }
  }, [presetValue, sourceType, isMultiple])

  const valueShow = useMemo(() => {
    if (isMultiple || isEdit) {
      return value ? value.split(',') : []
    } else {
      return value || null
    }
  }, [isEdit, isMultiple, value])

  const enums = useMemo(() => {
    switch (sourceType) {
      case '手动输入':
        return Array.isArray(options) ? options : []
      case '数据字典':
        return Array.isArray(dictionaryList) ? dictionaryList.map(v => v.nodeName) : []
      case '表数据':
        return Array.isArray(tableData) ? tableData : []
      case '站点':
        return Array.isArray(site) ? site : []
      default:
        return []
    }
  }, [sourceType, options, dictionaryList, tableData, site])

  const children = useMemo(() => {
    let children = []
    if (Array.isArray(enums)) {
      enums.forEach(v => {
        if (sourceType === '站点') {
          children.push(<Option key={isStoreID ? v.stationID : v.stationName} value={isStoreID ? v.stationID : v.stationName}>{v.stationName}</Option>)
        } else if (sourceType === '手动输入') {
          children.push(
            <Option key={v.value} value={v.value}>
              {v.value}
            </Option >
          )
        } else {
          children.push(<Option key={v} value={v}>{v}</Option>)
        }
      })
    }
    return children
  }, [sourceType, enums, isStoreID])

  const handleChange = (value) => {
    if (addons) {
      if (isMultiple || isEdit) {
        onChange(value.join(',') || '')
      } else {
        onChange(value || '')
      }
    }
  }

  const getStation = async (presetValue) => {
    const { code, data, msg } = await getStationIDListByUserID(userID, !isMySite)
    if (code === 0) {
      if (Array.isArray(data)) {
        let values = data.map(v => ({ stationID: v.stationID + '', stationName: v.stationName }))
        setSite(values)
        if (!value && !presetValue && !disabled) {
          onChange(values[0][isStoreID ? 'stationID' : 'stationName'])
        }
      } else {
        setSite([])
      }
    } else {
      message.error(msg)
    }
  }

  const onFocus = async () => {
    if (addons) {
      if (sourceType === '数据字典') {
        if (!dictionary) {
          return message.error('请选择数据字典！')
        }
        const { data } = await GetSelectItemList({ nodeName: dictionary })
        if (Array.isArray(data)) {
          setDictionaryList(data)
        }
      }
      if (sourceType === '表数据') {
        if (!tableName) {
          return message.info('请选择表名!')
        }
        if (!fieldName) {
          return message.info('请选择字段名!')
        }
        const { code, data, msg } = await GetFieldValueFromTable(tableName, fieldName)
        if (code === 0) {
          if (Array.isArray(data)) {
            setTableData(data.filter(v => v))
          } else {
            setTableData([])
          }
        } else {
          message.error(msg)
        }
      }
      if (sourceType === '站点') {
        getStation()
      }
    }
  }

  const tagRender = (props) => {

    const { label, value, closable, onClose } = props

    const color = useMemo(() => {
      let color = ''
      options.forEach(v => {
        if (v.value === value) {
          color = v.color
        }
      })
      return color
    }, [options, value])

    const onPreventMouseDown = (event) => {
      event.preventDefault()
      event.stopPropagation()
    }

    if (!color) {
      return label
    }

    return (
      <span style={{ marginRight: '3px' }}>
        <TagPack color={color} text={label} onClose={onClose} />
      </span>
    )
  }

  const isTagRender = color ? { tagRender } : {}

  return (
    <Select
      onFocus={onFocus}
      style={{ width: '100%' }}
      mode={isEdit ? 'tags' : (isMultiple ? 'multiple' : '')}
      disabled={disabled}
      showArrow={!disabled}
      showSearch={isSearch}
      value={valueShow}
      placeholder={disabled ? (placeholder || '') : (placeholder || '请选择内容')}
      onChange={handleChange}
      allowClear
      // getPopupContainer={(targterNode) => targterNode.parentElement || document.body}
      {...isTagRender}
    >
      {children}
    </Select>
  )

}

export default ComboBox
