import React, { useState, useEffect, useMemo } from 'react'
import { Radio } from 'antd'
import { GetSelectItemList, GetFieldValueFromTable, getStationListByUserID, getStationIDListByUserID } from '../../../../apis/process'
import TagPack from '../../../components/TagPack'

const RadioButton = (props) => {

  const { value, onChange, schema, addons } = props
  const { presetValue, disabled, sourceType, options, dictionary, tableName, fieldName, isStoreID, color } = schema
  const [dictionaryList, setDictionaryList] = useState([])
  const [tableData, setTableData] = useState([])
  const [site, setSite] = useState([])

  useEffect(() => {
    if (addons) {
      addons.setValue(addons.dataPath, presetValue || '')
    }
  }, [presetValue])

  const onFocus = async () => {
    if (addons) {
      if (sourceType === '数据字典') {
        if (!dictionary) {
          return message.error('请选择数据字典！')
        }
        const { data } = await GetSelectItemList({ nodeName: dictionary })
        if (Array.isArray(data)) {
          setDictionaryList(data)
        }
      }
      if (sourceType === '表数据') {
        if (!tableName) {
          return message.info('请选择表名!')
        }
        if (!fieldName) {
          return message.info('请选择字段名!')
        }
        const { code, data, msg } = await GetFieldValueFromTable(tableName, fieldName)
        if (code === 0) {
          if (Array.isArray(data)) {
            setTableData(data.filter(v => v))
          } else {
            setTableData([])
          }
        } else {
          message.error(msg)
        }
      }
      if (sourceType === '站点') {
        const { code, data, msg } = await getStationIDListByUserID(userID, !isMySite)
        if (code === 0) {
          if (Array.isArray(data)) {
            setSite(data)
          } else {
            setSite([])
          }
        } else {
          message.error(msg)
        }
      }
    }
  }

  const radioChange = (e) => {
    if (addons) {
      addons.setValue(addons.dataPath, e.target.value)
    }
  }

  useEffect(() => {
    if (sourceType === '数据字典' && dictionary) {
      onFocus()
    }
    if (sourceType === '表数据' && tableName && fieldName) {
      onFocus()
    }
  }, [sourceType, dictionary, tableName, fieldName])

  const enums = useMemo(() => {
    switch (sourceType) {
      case '手动输入':
        return Array.isArray(options) ? options : []
      case '数据字典':
        return Array.isArray(dictionaryList) ? dictionaryList.map(v => v.nodeName) : []
      case '表数据':
        return Array.isArray(tableData) ? tableData : []
      case '站点':
        return Array.isArray(site) ? site : []
      default:
        return []
    }
  }, [sourceType, options, dictionaryList])

  const children = useMemo(() => {
    let children = []
    if (Array.isArray(enums)) {
      enums.forEach(v => {
        if (sourceType === '站点') {
          children.push(<Radio key={isStoreID ? v.stationID : v.stationName} value={isStoreID ? v.stationID : v.stationName}>{v.stationName}</Radio>)
        } else if (sourceType === '手动输入') {
          children.push(
            <Radio key={v.key || v.value} value={v.value}>
              {
                color ? <TagPack color={v.color} text={v.value} /> : v.value
              }
            </Radio >
          )
        } else {
          children.push(<Radio key={v} value={v}>{v}</Radio>)
        }
      })
    }
    return children
  }, [sourceType, enums])

  return (
    <Radio.Group onChange={radioChange} value={value} disabled={disabled}>
      {children}
    </Radio.Group>
  )

}

export default RadioButton