import React, { useMemo, useState, useContext } from 'react'
import { Input, Modal, ConfigProvider } from 'antd'
import Icon, { PlusOutlined } from '@ant-design/icons'
import * as icons from '@ant-design/icons'
import styles from './index.less'
import Drag from '../../../components/Drag'

const iconList = Object.keys(icons).filter((item) => typeof icons[item] === 'object')

const InputAddon = (props) => {

  const { value, onChange, disabled } = props

  const [visible, setVisible] = useState(false)
  const [icon, setIcon] = useState()

  const inputChange = (e) => {
    onChange(e.target.value)
  }

  const onOk = () => {
    setVisible(false)
    onChange(icon)
  }

  const iconClick = () => {
    setVisible(true)
  }

  const onCancel = () => {
    setVisible(false)
  }

  return (
    <div className={styles.inputAddon}>
      <Input disabled={disabled} value={value} onChange={inputChange} />
      {!disabled ? <div className={styles.addon}><PlusOutlined onClick={iconClick} /></div> : null}
      <Drag
        width={'50%'}
        getContainer={false}
        title='图标选择'
        visible={visible}
        onOk={onOk}
        onCancel={onCancel}
        bodyStyle={{ height: '450px', overflow: 'auto' }}
      >
        <div className={styles.iconBox}>
          {
            iconList.map((v, i) => {
              if (i === 0) return null
              return (
                <div key={v} className={`iconItem`} active={JSON.stringify(icon === v)} onClick={() => setIcon(v)}>
                  <Icon component={icons[v]} />
                </div>
              )
            })
          }
        </div>
      </Drag>
    </div>
  )

}

export default InputAddon