import React, { useMemo } from 'react'
import { Cascader } from 'antd'
import city from '../../../../../constant/city'

const CascadeDefault = (props) => {

  const { value, onChange, schema, addons } = props
  const { sourceType } = addons.formData

  const enums = useMemo(() => {
    if (sourceType === '城市数据') {
      return city
    }
    return []
  }, [sourceType])


  const getDefaultValue = selectedValues => {
    if (!selectedValues || !selectedValues.length) {
      return []
    }

    const result = []
    // 递归查询城市名称数据
    function findLabel(children) {
      children.forEach(item => {
        const { label, value, children } = item;
        if (selectedValues.includes(label)) {
          result.push(value);
        }
        if (children && children.length) {
          findLabel(children);
        }
      });
    }
    // 省份解决县或市名称一样的问题
    city.forEach(item => {
      const { label, value, children } = item;
      if (selectedValues.includes(label)) {
        result.push(value);
        if (children && children.length) {
          findLabel(children);
        }
      }
    });
    // 设置默认值
    return result;
  }

  const cascaderChange = selectedValues => {
    if (!selectedValues || !selectedValues.length) {
      return onChange('')
    }
    const result = []
    // 递归查询城市所有数据
    function findCityData(city) {
      city.forEach(item => {
        const { id, label, value, children } = item;
        if (selectedValues.includes(value)) {
          result.push({ id, label, value })
          if (children && children.length) {
            findCityData(children)
          }
        }
      })
    }
    findCityData(city)
    // 分发监听
    const cityValue = result.map(item => {
      return item.label;
    })
    onChange(cityValue.join('/'));
  }

  return (
    <Cascader
      value={value}
      showSearch
      onChange={cascaderChange}
      style={{ width: '100%' }}
      options={enums}
    />
  )

}

export default CascadeDefault