import React, { useContext, useEffect, useMemo, useState } from 'react'
import { Input, Select, message, Radio, Row, Col } from 'antd'
import Drag from '../../../../components/Drag'
import { PlusOutlined } from '@ant-design/icons'
import { LoadTableFields, ReloadTableFields } from '../../../../../apis/process'
import styles from './index.less'
import { widgetData, getStyles } from '../../../../../constant'
import { getNanoid } from '../../../../../utils/index'

const FieldNames = (props) => {

  let fields = window.designer || []
  const { value, onChange, addons, disabled } = props
  const { tableNameParent, widget, $id } = addons.formData
  const isRelevanceFieldParent = widget === 'RelevanceSelect' && addons.dataPath === 'fieldParent'
  const [fieldName, setFieldName] = useState([])
  const [visible, setVisible] = useState(false)
  const [radioValue, setRadioValue] = useState(value)

  const options = useMemo(() => {
    if (isRelevanceFieldParent) {
      return fieldName
    } else {
      let array = []
      fieldName?.forEach(v => {
        let item = fields.find(s => s.fieldName === v.name) || {}
        array.push({ ...v, ...item })
      })
      return array
    }
  }, [fieldName, addons])

  const plusShow = useMemo(() => {
    if (isRelevanceFieldParent) {
      return false
    }
    return true
  }, [addons, widget])

  const selectChange = (value) => {
    if (addons.dataPath === '$id') {
      if (widget !== 'RelationForm') {
        addons.setValueByPath('title', value)
      } else {
        addons.setValueByPath('titleShow', value)
      }
    }
    onChange(value)
  }

  const radioChange = (e) => {
    setRadioValue(e.target.value)
  }

  const onFocus = async () => {
    if (!tableNameParent) {
      return message.info('缺少表名!')
    }
    if (isRelevanceFieldParent) {
      //防止没有选择数据源情况
      if ($id && $id.includes('关联选择_') && $id.length === 11) {
        return message.info('请先选择数据源')
      }
    }
    const { data } = await ReloadTableFields({ tableName: tableNameParent })
    if (Array.isArray(data.root)) {
      let array = []
      if (isRelevanceFieldParent) {
        array = data.root.filter(v => v.name !== $id)
      } else {
        array = data.root
      }
      setFieldName(array)
    }
  }

  const iconClick = () => {
    onFocus()
    setRadioValue(value)
    setVisible(true)
  }

  const onOk = () => {
    onChange(radioValue)
    if (widget !== 'RelationForm') {
      addons.setValueByPath('title', radioValue)
    } else {
      addons.setValueByPath('titleShow', radioValue)
    }
    setVisible(false)
  }

  useEffect(() => {
    if (value && value.includes('#/')) {
      let array = value.split('/')
      if (array.length === 3) {
        if (array[2].includes('_')) {
          let copy = array[2].split('_')
          if (copy.length > 3 && copy[1].length === 6) {
            let nanoid = getNanoid(6, `${copy[0]}_`)
            onChange(nanoid)
          } else {
            onChange(array[2])
          }
        } else {
          onChange(array[2])
        }
      } else if (array.length === 2) {
        onChange(array[1])
      }
    }
  }, [value])

  return (
    <div className={styles.fieldNames}>
      <Select
        disabled={disabled}
        onFocus={onFocus}
        onChange={selectChange}
        style={{ width: '100%' }}
        showSearch
        optionFilterProp='children'
        value={value}
        suffixIcon={null}
        getPopupContainer={(targterNode) => targterNode.parentElement || document.body}
      >
        {
          options.map(v => {
            if (!v.widget) {
              return <Select.Option key={v.name} value={v.name}>{v.name}</Select.Option>
            }
            return null
          })
        }
      </Select>
      {
        plusShow && !disabled ? (
          <div className={styles.plus}>
            <PlusOutlined onClick={iconClick} />
          </div>
        ) : null
      }
      <Drag
        title='字段配置'
        width='50%'
        visible={visible}
        onCancel={() => setVisible(false)}
        onOk={onOk}
        bodyStyle={{ height: '500px', overflow: 'auto' }}
      >
        <Radio.Group onChange={radioChange} value={radioValue} style={{ width: '100%' }}>
          <div className={styles.groupBox} style={{ display: options.filter(v => !v.widget).length ? 'block' : 'none' }}>
            <div className={styles.boxTitle} style={{ color: 'red' }}>未配置字段</div>
            <Row>
              {
                options.map(v => {
                  if (!v.widget) {
                    return (
                      <Col key={v.name} span={8} style={{ marginTop: '10px' }}>
                        <Radio key={v.name} value={v.name}>{v.name}</Radio>
                      </Col>
                    )
                  }
                  return null
                })
              }
            </Row>
          </div>
          <div className={styles.groupBox} style={{ marginTop: '20px', display: options.filter(v => v.widget).length ? 'block' : 'none' }}>
            <div className={styles.boxTitle} style={{ color: '#40a9ff' }}>已配置字段</div>
            <Row>
              {
                options.map(v => {
                  if (v.widget) {
                    return (
                      <Col span={8} key={v.name} style={{ marginTop: '15px', display: 'flex', justifyContent: 'space-between' }}>
                        <Radio disabled key={v.name} value={v.name}>
                          {v.name}
                        </Radio>
                        <div
                          style={{
                            fontSize: '12px',
                            borderRadius: '5px',
                            padding: '0 7px',
                            border: '1px solid',
                            marginRight: '20px',
                            ...getStyles(widgetData[v.widget]?.type)
                          }}
                        >{widgetData[v.widget]?.name}</div>
                      </Col>
                    )
                  }
                  return null
                })
              }
            </Row>

          </div>
        </Radio.Group>
      </Drag>
    </div>
  )

}

export default FieldNames