import React, { useState, useEffect, useRef, useImperativeHandle, forwardRef, useMemo, useLayoutEffect } from 'react'
import { Input } from 'antd'
import styles from './index.less'
import { request } from '@wisdom-utils/utils'
import { Editor, Toolbar } from '@wangeditor/editor-for-react'
import { DomEditor } from '@wangeditor/editor'
import '@wangeditor/editor/dist/css/style.css'
import Drag from '../../../../components/Drag'

const uploadFileAPI = `/PandaWorkFlow/WorkFlow/AccountManage/UploaderFiles`;
const downFileAPI = `/PandaWorkFlow/WorkFlow/AccountManage/DownloadFiles`;

const RichTextDefault = (props, ref) => {

  const { schema, onChange, value, addons } = props
  const { title } = addons?.formData || {}
  const { disabled, placeholder } = schema
  const [text, setText] = useState('')

  const [visible, setVisible] = useState(false)
  const [editor, setEditor] = useState(null)

  const editorChange = (editor) => {
    setText(editor?.getHtml())
  }

  const iconClick = () => {
    setVisible(true)
  }

  const onOk = () => {
    onChange(text)
    setVisible(false)
  }

  useEffect(() => {
    return () => {
      if (editor !== null) {
        editor.destroy()
        setEditor(null)
      }
    }
  }, [editor])

  return (
    <div className={styles.richTextDefault}>
      <Input style={{ color: 'blue' }} value={value ? '点击查看详情' : ''} onClick={iconClick} />
      <Drag
        width='60%'
        title={`${title}默认值`}
        onCancel={() => setVisible(false)}
        visible={visible}
        onOk={onOk}
        getContainer={document.body}
      >
        <div className={styles.richText}>
          <Toolbar
            editor={editor}
            defaultConfig={{
              excludeKeys: ['insertLink', 'group-image', 'group-video', 'codeBlock', 'undo', 'redo', 'fullScreen']
            }}
            mode="default"
            style={{ borderBottom: '1px solid #ccc' }}
          />
          <Editor
            defaultConfig={{
              readOnly: disabled,
              placeholder: placeholder || '',
              focus: false,
              pasteFilterStyle: true, // 忽略粘贴样式
              pasteIgnoreImg: true, // 忽略粘贴的图片
              styleWithCSS: false,
              zIndex: 500,
              MENU_CONF: {
                uploadImage: {
                  customInsert(res, inserFn) {
                    const formData = new FormData();
                    formData.append('file', file);
                  },
                  server: uploadFileAPI,
                }
              }
            }}
            value={value}
            onCreated={setEditor}
            onChange={editorChange}
            mode="default"
            style={{ minHeight: '300px', overflowY: 'hidden' }}
          />
        </div>
      </Drag>
    </div>
  )

}

export default forwardRef(RichTextDefault);
