import React, { useState, useRef } from 'react'
import { Modal, Input, message, Dropdown, Menu, Tree } from 'antd'
import styles from './index.less'
import { getTableNumberList, getFormFieldAnalysisDtos } from '../../../../../apis/process'
import { isObject } from '../../../../../utils'
import Drag from '../../../../components/Drag'

const { TextArea } = Input

const fnList = [
  { label: '最大值', value: '$sql.max()' },
  { label: '最小值', value: '$sql.min()' },
  { label: '求和', value: '$sql.sum()' },
  { label: '平均值', value: '$sql.avg()' },
  { label: '时间差', value: '$sql.datediff()' },
  { label: '向下取整', value: '$sql.ceiling()' },
  { label: '向上取整', value: '$sql.floor()' },
]

const CalculateRule = (props) => {

  const { value, onChange, addons } = props
  const { tableNameParent, widget } = addons.formData
  const [visible, setVisible] = useState(false)
  const [fieldList, setFieldList] = useState([])
  const [rule, setRule] = useState([])
  const [currentSelectId, setCurrentSelectId] = useState('')
  const ruleText = useRef()

  const getFieldList = async () => {
    let { code, data, msg } = await getTableNumberList(tableNameParent)
    if (code === 0) {
      let array = []
      if (isObject(data)) {
        for (let key in data) {
          array.push({
            title: key,
            key: key,
            children: Array.isArray(data[key]) ? data[key].map(v => { return { title: `${v.Name}(${v.Shape})`, key: `${key}.${v.Name}` } }) : []
          })
        }
      }
      setFieldList(array)
      if (array.length > 0) {
        setVisible(true)
      } else {
        message.error('无可用字段')
      }
    } else {
      message.error(msg)
    }
  }

  const inputClick = () => {
    getFieldList()
    setRule(value || '')
    if (value) {
      setTimeout(() => {
        document
          .getElementById('calculateRule')
          .setSelectionRange(value.length, value.length)
      }, 200)
    }
  }

  const onSelect = (prop, treeNode) => {
    if (!treeNode.node.children) {
      insert(`{${treeNode.node.key}}`)
    }
    setCurrentSelectId(prop[0]);
  }

  const onOk = async () => {
    if (rule) {
      const { code, data, msg } = await getFormFieldAnalysisDtos({ formula: rule })
      if (code === 0) {
        addons.setValue('rules', data)
        setVisible(false)
        onChange(rule)
        if (widget === 'NumberInput') {
          addons.setValue('disabled', true)
        }
      } else {
        message.error(msg)
      }
    } else {
      addons.setValue('rules', [])
      setVisible(false)
      onChange(rule)
    }
  }

  const onCancel = () => {
    setVisible(false)
  }

  const insert = text => {
    let ednIndex = document.getElementById('calculateRule').selectionEnd
    let rangeIndex = document.getElementById('calculateRule').selectionStart
    setRule(`${rule.slice(0, rangeIndex)}${text}${rule.slice(rangeIndex)}`)
    rangeIndex += text.toString().length
    ruleText.current.focus()
  }

  const insertFn = value => {
    insert(value)
  }

  const fnListRender = (
    <Menu>
      {fnList.map(item => (
        <Menu.Item key={item.value} onClick={() => insertFn(item.value)}>
          {item.label}
        </Menu.Item>
      ))}
    </Menu>
  )

  return (
    <div>
      <Input
        value={value}
        onClick={inputClick}
      />
      <Drag
        title="计算规则"
        visible={visible}
        onOk={onOk}
        width="740px"
        onCancel={onCancel}
        maskClosable={false}
        destroyOnClose
        getContainer={false}
      >
        <div className={styles.configContent}>
          <div className={styles.leftTree}>
            <Tree
              blockNode
              defaultExpandedKeys={[tableNameParent]}
              autoExpandParent={true}
              onSelect={onSelect}
              selectedKeys={[currentSelectId]}
              treeData={fieldList}
            />
          </div>
          <div className={styles.rightContent}>
            <div className={styles.textAreaBox}>
              <Dropdown overlay={fnListRender} placement="bottom" arrow>
                <div className={styles.insertFn}>插入函数</div>
              </Dropdown>
              <TextArea
                id="calculateRule"
                autoSize={{ minRows: 19, maxRows: 19 }}
                value={rule}
                rows={4}
                ref={ruleText}
                onChange={e => setRule(e.target.value)}
              />
            </div>
            <ul className={styles.tipBox}>
              <li>
                <p>请从左侧面板选择字段或选项</p>
                <p>
                  {'支持'} <i>英文</i> {'模式下运算符（+、-、*、/）'}
                </p>
              </li>
              <li>{`示例：{表名.缴费次数} + {表名.缴费金额}`}</li>
            </ul>
          </div>
        </div>
      </Drag>
    </div>
  )

}

export default CalculateRule
