import React, { useState, useRef } from 'react'
import { Modal, Input, message, Dropdown, Menu, Tree } from 'antd'
import styles from './index.less'
import { ReloadTableFields, ruleValidation } from '../../../../../apis/process'
import Drag from '../../../../components/Drag'

const { TextArea } = Input

const fnList = [
  { label: '是否为工作日', value: '$fn.isWorkDay()' },
  { label: '是否为工作时间', value: '$fn.isWorkTime()' },
]

const ControlRules = (props) => {

  const { value, onChange, addons } = props
  const { tableNameParent } = addons.formData
  const [visible, setVisible] = useState(false)
  const [fieldList, setFieldList] = useState([])
  const [rule, setRule] = useState([])
  const [currentSelectId, setCurrentSelectId] = useState('')
  const ruleText = useRef()

  const getFieldList = async () => {
    let { code, data, msg } = await ReloadTableFields({ tableName: tableNameParent })
    if (code === 0) {
      setFieldList([
        {
          title: tableNameParent,
          key: tableNameParent,
          children: data.root.map(v => { return { title: v.name, key: `${tableNameParent}.${v.name}` } })
        }
      ])
    } else {
      message.error(msg)
    }
  }

  const inputClick = () => {
    setVisible(true)
    getFieldList()
    setRule(value || '')
    if (value) {
      setTimeout(() => {
        document
          .getElementById('ruleText')
          .setSelectionRange(value.length, value.length)
      }, 200)
    }
  }

  const onSelect = (prop, treeNode) => {
    if (!treeNode.node.children) {
      insert(`{${treeNode.node.key}}`)
    }
    setCurrentSelectId(prop[0]);
  }

  const onOk = async () => {
    if (rule) {
      const { code, data } = await ruleValidation(rule)
      if (!data) {
        return message.error('校验不通过!')
      }
    }
    onChange(rule)
    setVisible(false)
  }

  const onCancel = () => {
    setVisible(false)
  }

  const insert = text => {
    let ednIndex = document.getElementById('ruleText').selectionEnd
    let rangeIndex = document.getElementById('ruleText').selectionStart
    setRule(`${rule.slice(0, rangeIndex)}${text}${rule.slice(rangeIndex)}`)
    rangeIndex += text.toString().length
    ruleText.current.focus()
  }

  const insertFn = value => {
    insert(value)
  }

  const fnListRender = (
    <Menu>
      {fnList.map(item => (
        <Menu.Item key={item.value} onClick={() => insertFn(item.value)}>
          {item.label}
        </Menu.Item>
      ))}
    </Menu>
  )

  return (
    <div>
      <Input
        value={value}
        onClick={inputClick}
      />
      <Drag
        title="规则配置"
        visible={visible}
        onOk={onOk}
        width="740px"
        onCancel={onCancel}
        maskClosable={false}
        destroyOnClose
        getContainer={false}
      >
        {
          tableNameParent ? (
            <div className={styles.configContent}>
              <div className={styles.leftTree}>
                <Tree
                  blockNode
                  defaultExpandedKeys={[tableNameParent]}
                  autoExpandParent={true}
                  onSelect={onSelect}
                  selectedKeys={[currentSelectId]}
                  treeData={fieldList}
                />
              </div>
              <div className={styles.rightContent}>
                <div className={styles.textAreaBox}>
                  <Dropdown overlay={fnListRender} placement="bottom" arrow>
                    <div className={styles.insertFn}>插入函数</div>
                  </Dropdown>
                  <TextArea
                    id="ruleText"
                    autoSize={{ minRows: 19, maxRows: 19 }}
                    value={rule}
                    rows={4}
                    ref={ruleText}
                    onChange={e => setRule(e.target.value)}
                  />
                </div>

                <ul className={styles.tipBox}>
                  <li>
                    <p>请从左侧面板选择字段或选项</p>
                    <p>
                      {'支持'} <i>英文</i> {'模式下运算符（+、-、*、/、>、<、=、!=、<=、>=）'}
                    </p>
                  </li>
                  <li>{`示例：{表名.是否缴费} = ‘是’ and {表名.缴费金额} > 1000`}</li>
                </ul>
              </div>
            </div>
          ) : null
        }
      </Drag>
    </div>
  )

}

export default ControlRules
