import React, { useEffect, useState } from 'react'
import { Form, Space, TreeSelect, Button, Input, Modal, message, Select } from 'antd'
import { MinusCircleOutlined, PlusOutlined } from '@ant-design/icons'
import styles from './index.less'
import { ReloadTableFields, QueryFields, getMaplayerByTerminalType, GetLayerList, GetLayerFields } from '../../../../../apis/process'
import Drag from '../../../../components/Drag'

const { TreeNode } = TreeSelect

const DeviceFieldshine = (props) => {

  const { addons, onChange, value } = props
  const formData = addons.formData

  const [form] = Form.useForm()
  const [visible, setVisible] = useState(false)
  const [mapServers, setMapServers] = useState([])
  const [service, setService] = useState(formData?.service)
  const [layers, setLayers] = useState([])
  const [layerName, setlayerName] = useState(formData?.layerName)
  const [fromField, setFromField] = useState([])
  const [toField, setToField] = useState([])

  const onOk = () => {
    form
      .validateFields()
      .then((values) => {
        const { field, service, layerName } = form.getFieldsValue()
        addons.setValue('service', service)
        addons.setValue('layerName', layerName)
        onChange(field)
        setVisible(false)
      })
      .catch((info) => {
        console.log('Validate Failed:', info)
      })
  }

  const inputClick = async () => {
    const { code, data } = await ReloadTableFields({ tableName: formData?.tableNameParent })
    if (code === 0) {
      setToField(data.root)
    }
    let field = value.length ? value : []
    form.setFieldsValue({ field, service: formData?.service, layerName: formData?.layerName })
    setVisible(true)
  }

  const serviceFocus = async () => {
    const { code, data } = await getMaplayerByTerminalType()
    if (code === '0') {
      setMapServers(data)
    }
  }

  const layerNameFocus = async () => {
    let { success, root } = await GetLayerList(service)
    if (success === true) {
      let array = []
      root.forEach(v => {
        let values = v.layerName.split(':')
        if (values.length === 2) {
          array.push({ layerID: v.layerID, layerName: values[1] })
        }
      })
      setLayers(array)
    }
  }

  const fromFieldFocus = async (value) => {
    const { code, data } = await GetLayerFields({ mapServer: service, layerName: layerName })
    if (code === '0') {
      setFromField(data)
    }
  }

  const fromFieldChange = (value) => {
    const { field, service, layerName } = form.getFieldsValue()
    let values = field.map(v => ({ ...v, layerName: v.fromField === value ? layerName : v.layerName }))
    form.setFieldsValue({ field: values, service, layerName })
  }

  const clear = () => {
    addons.setValue('service', '')
    addons.setValue('layerName', '')
    onChange([])
    setVisible(false)
  }

  return (
    <div className={styles.fieldshine}>
      <Input
        value={value && value.length ? JSON.stringify(value) : null}
        onClick={inputClick}
      />
      <Drag
        getContainer={false}
        width='450px'
        title='字段映射'
        visible={visible}
        footer={
          <div>
            <Button onClick={() => setVisible(false)}>取消</Button>
            <Button onClick={() => clear()}>清空</Button>
            <Button type="primary" onClick={() => onOk()}>确定</Button>
          </div>
        }
        onCancel={() => setVisible(false)}
      >
        <Form name="dynamic_form_nest_item" form={form} autoComplete="off">
          <Form.Item
            label='方案名称'
            name='service'
            rules={[
              {
                required: true,
                message: '方案名称必填',
              },
            ]}
          >
            <Select
              onFocus={serviceFocus}
              onChange={(value) => setService(value)}
            >
              {
                mapServers.map(v => <Select.Option value={v.servicename}>{v.schemename}</Select.Option>)
              }
            </Select>
          </Form.Item>
          <Form.Item
            label='图层名称'
            name='layerName'
            rules={[
              {
                required: true,
                message: '图层名称必填',
              },
            ]}
          >
            <Select
              onFocus={layerNameFocus}
              onChange={(value) => setlayerName(value)}
            >
              {
                layers.map(v => <Select.Option value={v.layerName}>{v.layerName}</Select.Option>)
              }
            </Select>
          </Form.Item>
          <Form.List name="field" label='字段映射'>
            {(value, { add, remove }) => (
              <>
                {value.map(({ key, name, ...restField }, index) => (
                  <Space
                    key={key}
                    style={{
                      display: 'flex',
                      marginBottom: '5px',
                      justifyContent: 'center',
                      position: 'relative',
                      left: index === 0 ? '-11px' : '',
                    }}
                    align="baseline"
                  >
                    <Form.Item
                      {...restField}
                      name={[name, 'fromField']}
                      rules={[
                        {
                          required: true,
                          message: '来源字段必填',
                        }
                      ]}
                    >
                      <TreeSelect
                        showSearch
                        style={{ width: '180px' }}
                        dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                        placeholder="请选择字段名 "
                        allowClear
                        treeDefaultExpandAll
                        showCheckedStrategy
                        onFocus={fromFieldFocus}
                        onChange={fromFieldChange}
                      >
                        {fromField.map(s => <TreeNode value={s.field} title={s.fieldAlias} key={s.field} />)}
                      </TreeSelect>
                    </Form.Item>
                    <div className={styles.centerImg}></div>
                    <Form.Item
                      {...restField}
                      name={[name, 'toField']}
                      rules={[
                        {
                          required: true,
                          message: '目标字段必填',
                        },
                        {
                          validator: (rules, value, callback) => {
                            const { field } = form.getFieldsValue()
                            let toFields = Array.from(new Set([...field.map(v => v.toField)]))
                            if (toFields.length !== field.length) {
                              return Promise.reject('目标字段不能重复！')
                            }
                            return Promise.resolve()
                          }
                        }
                      ]}
                    >
                      <TreeSelect
                        showSearch
                        style={{ width: '180px' }}
                        dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                        placeholder="请选择字段名 "
                        allowClear
                        treeDefaultExpandAll
                        showCheckedStrategy
                      >
                        <TreeNode value={formData?.tableNameParent} title={formData?.tableNameParent} disabled>
                          {toField.map(s => <TreeNode value={s.name} title={s.name} key={s.name} />)}
                        </TreeNode>
                      </TreeSelect>
                    </Form.Item>
                    <Form.Item
                      hidden={true}
                      {...restField}
                      name={[name, 'layerName']}
                    >
                      <Input />
                    </Form.Item>
                    {index !== 0 ? <MinusCircleOutlined onClick={() => remove(name)} /> : null}
                  </Space>
                ))}
                <Form.Item style={{ padding: '0 10px' }}>
                  <Button type="dashed" onClick={() => add()} block icon={<PlusOutlined />}>
                    添加映射
                  </Button>
                </Form.Item>
              </>
            )}
          </Form.List>
        </Form>
      </Drag>
    </div>
  )

}

export default DeviceFieldshine