import React, { useState } from 'react'
import { Input, Modal, Select, Button, Table } from 'antd'
import { EyeOutlined } from '@ant-design/icons'
import './index.less'
import { GetDataDictionaryList, GetSelectItemList } from '../../../../../apis/process'
import Drag from '../../../../components/Drag'

const columns = [
  {
    title: '序号',
    render: (t, r, i) => i + 1,
    width: 60
  },
  {
    title: '名称',
    dataIndex: 'nodeName',
  },
  {
    title: '值',
    dataIndex: 'nodeValue',
  }
]

const Dictionary = (props) => {

  const { value, shame, onChange } = props

  const [options, setOptions] = useState([])
  const [visible, setVisible] = useState(false)
  const [dataSource, setDataSource] = useState([])

  const iconClick = () => {
    setVisible(true)
    getDataSource()
  }

  const getDataSource = async () => {
    const { data } = await GetSelectItemList({ nodeName: value })
    if (Array.isArray(data)) {
      setDataSource(data)
    }
  }

  const getDictionary = async () => {
    const { data } = await GetDataDictionaryList({})
    if (Array.isArray(data)) {
      setOptions(data)
      return data
    }
    return []
  }

  const selectChange = (value) => {
    onChange(value)
  }

  return (
    <div className='dictionary'>
      <Select
        value={value}
        style={{ width: '100%' }}
        onChange={selectChange}
        onFocus={() => getDictionary()}
        showSearch
      // allowClear
      >
        {
          options.map(v => {
            return (
              <Select.Option key={v.nodeName} value={v.nodeName} >
                {v.nodeName}
              </Select.Option>
            )
          })
        }
      </Select>
      <div className='dictionary-icons' style={{ display: value ? 'block' : 'none' }}><EyeOutlined onClick={iconClick} /></div>
      <Drag
        width={'50%'}
        onCancel={() => setVisible(false)}
        onOk={() => setVisible(false)}
        title={'数据字典' + `【${value}】`}
        visible={visible}
        getContainer={false}
      >
        <Table
          size='small'
          bordered
          dataSource={dataSource}
          columns={columns}
        />
      </Drag>
    </div>
  )

}

export default Dictionary