import React, { useMemo, useState } from 'react'
import { Input, message, Row, Col, Checkbox } from 'antd'
import Drag from '../../../../components/Drag'
import {
  GetAccountPageList,
  GetAccountConfigInfo,
  GetTableGroupMeta,
  GetTableDataInfo,
  SaveTableDataInfo,
  EditTableDataInfo,
  DeleteTableDataInfo
} from '../../../../../apis/process'

const FieldList = (props) => {

  const { value, addons, onChange } = props
  const { accountName } = addons.formData
  const [visible, setVisible] = useState(false)
  const [conifg, setConfig] = useState({})

  const options = useMemo(() => {
    const { webShowFieldGroup } = conifg
    return webShowFieldGroup ? webShowFieldGroup.split(',').filter(v => v) : []
  }, [conifg])

  const getConfig = async () => {
    const { code, data } = await GetAccountConfigInfo(accountName)
    if (code === 0) {
      setConfig(data)
    }
  }

  const onOk = () => {
    setVisible(false)
  }

  const checkChange = (value) => {
    onChange(value)
  }

  const inputClick = () => {
    if (!accountName) {
      return message.error('请配置台账名称!')
    }
    getConfig()
    setVisible(true)
  }

  return (
    <div>
      <Input value={value} onClick={inputClick} />
      <Drag
        title='前端显示字段'
        onOk={onOk}
        onCancel={() => setVisible(false)}
        visible={visible}
      >
        <Checkbox.Group
          value={value}
          style={{ width: '100%' }}
          onChange={checkChange}
        >
          <Row>
            {
              options.map(v => {
                return (
                  <Col span={8}>
                    <Checkbox value={v}>{v}</Checkbox>
                  </Col>
                )
              })
            }
          </Row>
        </Checkbox.Group>
      </Drag>
    </div>
  )

}

export default FieldList