import React, { useEffect, useState } from 'react'
import { Form, Space, TreeSelect, Button, Input, Modal, message } from 'antd'
import { MinusCircleOutlined, PlusOutlined } from '@ant-design/icons'
import styles from './index.less'
import { ReloadTableFields, QueryFields, QueryAddFields } from '../../../../../apis/process'
import Drag from '../../../../components/Drag'

const { TreeNode } = TreeSelect

const MappedField = (props) => {

  const { value, onChange, schema, addons } = props
  const { '台账名称': accountName, tableNameParent } = addons.formData
  const [form] = Form.useForm()
  const [visible, setVisible] = useState(false)
  const [fromField, setFromField] = useState([])
  const [toField, setToField] = useState([])

  const inputClick = async () => {
    if (!accountName) {
      message.info('请选择台账名称')
    }
    const req1 = QueryAddFields(accountName)
    const req2 = ReloadTableFields({ tableName: tableNameParent })
    const [res1, res2] = await Promise.all([req1, req2])
    if (res1.code === 0) {
      setToField(res1.data)
    } else {
      message.error(res1.msg)
    }
    if (res2.code === 0) {
      setFromField(res2.data.root)
    } else {
      message.error(res2.msg)
    }
    form.setFieldsValue({ field: value })
    setVisible(true)
  }

  const onOk = () => {
    form
      .validateFields()
      .then((values) => {
        const { field } = form.getFieldsValue()
        onChange(field)
        setVisible(false)
      })
      .catch((info) => {
        console.log('Validate Failed:', info)
      })
  }

  return (
    <div className={styles.mappedField}>
      <Input
        value={value && value.length ? JSON.stringify(value) : null}
        onClick={inputClick}
      />
      <Drag
        title={accountName}
        visible={visible}
        onCancel={() => setVisible(false)}
        onOk={onOk}
      >
        <Form name="dynamic_form_nest_item" form={form} autoComplete="off">
          <Form.List name="field">
            {(value, { add, remove }) => (
              <>
                {value.map(({ key, name, ...restField }, index) => (
                  <Space
                    key={key}
                    style={{
                      display: 'flex',
                      marginBottom: '5px',
                      justifyContent: 'center',
                    }}
                    align="baseline"
                  >
                    <Form.Item
                      {...restField}
                      name={[name, 'fromField']}
                      rules={[
                        {
                          required: true,
                          message: '来源字段必填',
                        }
                      ]}
                    >
                      <TreeSelect
                        showSearch
                        style={{ width: '180px' }}
                        dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                        placeholder="请选择字段名 "
                        allowClear
                        treeDefaultExpandAll
                        showCheckedStrategy
                      >
                        <TreeNode value='内置字段' title='内置字段' disabled>
                          {['工单编号', '事件编号'].map(s => <TreeNode value={s} title={s} key={s} />)}
                        </TreeNode>
                        <TreeNode value='附加字段' title='附加字段' disabled>
                          {fromField.map(s => {
                            if (['工单编号', '事件编号'].includes(s.name)) {
                              return null
                            }
                            return <TreeNode value={s.name} title={s.name} key={s.name} />
                          })}
                        </TreeNode>
                      </TreeSelect>
                    </Form.Item>
                    <div className={styles.centerImg}></div>
                    <Form.Item
                      {...restField}
                      name={[name, 'toField']}
                      rules={[
                        {
                          required: true,
                          message: '目标字段必填',
                        },
                        {
                          validator: (rules, value, callback) => {
                            const { field } = form.getFieldsValue()
                            let toFields = Array.from(new Set([...field.map(v => v.toField)]))
                            if (toFields.length !== field.length) {
                              return Promise.reject('目标字段不能重复！')
                            }
                            return Promise.resolve()
                          }
                        }
                      ]}
                    >
                      <TreeSelect
                        showSearch
                        style={{ width: '180px' }}
                        dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                        placeholder="请选择字段名 "
                        allowClear
                        treeDefaultExpandAll
                        showCheckedStrategy
                      >
                        <TreeNode value={accountName} title={accountName} disabled>
                          {toField.map(s => <TreeNode value={s} title={s} key={s} />)}
                        </TreeNode>
                      </TreeSelect>
                    </Form.Item>
                    <MinusCircleOutlined onClick={() => remove(name)} />
                  </Space>
                ))}
                <Form.Item style={{ padding: '0 10px' }}>
                  <Button type="dashed" onClick={() => add()} block icon={<PlusOutlined />}>
                    添加映射
                  </Button>
                </Form.Item>
              </>
            )}
          </Form.List>
        </Form>
      </Drag>
    </div>
  )

}

export default MappedField