import React, { useState } from 'react'
import { Form, Input, Select, Space, Button, TreeSelect } from 'antd'
import styles from './index.less'
import { MinusCircleOutlined, PlusOutlined } from '@ant-design/icons'
import Drag from '../../../../components/Drag'
import { LoadLedgers, QueryFields, ReloadTableFields } from '../../../../../apis/process'

const { TreeNode } = TreeSelect

const OtherSource = (props) => {

  const { addons, onChange, value } = props
  const { '台账名称': accountName } = addons?.formData || {}
  const [visible, setVisible] = useState(false)
  const [options, setOptions] = useState([])
  const [fromField, setFromField] = useState([])
  const [toField, setToField] = useState([])
  const [form] = Form.useForm()

  const toFieldFocus = async () => {
    const { code, data } = await QueryFields(accountName)
    if (code === 0) {
      setToField(data)
    }
  }

  const fromFieldFocus = async () => {
    const { accountName } = form.getFieldsValue()
    const { code, data } = await QueryFields(accountName)
    if (code === 0) {
      setFromField(data)
    }
  }

  const focusSelect = async () => {
    const { code, data, msg } = await LoadLedgers()
    if (code === 0) {
      if (Array.isArray(data.root)) {
        setOptions(data.root)
      } else {
        setOptions([])
      }
    } else {
      message.error(msg)
    }
  }

  const onOk = () => {
    setVisible(true)
    const values = form.getFieldsValue()
    onChange(values)
    setVisible(false)
  }

  const inputClick = () => {
    setVisible(true)
    form.setFieldsValue(value)
  }

  return (
    <div className={styles.otherSource}>
      <Input value={value ? JSON.stringify(value) : ''} onClick={inputClick} />
      <Drag
        onOk={onOk}
        onCancel={() => setVisible(false)}
        title={'数据来源'}
        visible={visible}
      >
        <Form form={form} autoComplete='off'>
          <Form.Item
            label={'台账名称'}
            name={'accountName'}
          >
            <Select showSearch allowClear onFocus={focusSelect} onChange={() => form.setFieldsValue({ ...form.getFieldsValue(), field: [] })}>
              {
                options.map(v => {
                  return (
                    <Select.Option key={v.name} value={v.name}>{v.name}</Select.Option>
                  )
                })
              }
            </Select>
          </Form.Item>
          <Space align="baseline">
            <div style={{ width: '60px' }}>字段映射:</div>
            <Form.List name="field" label='字段映射'>
              {(value, { add, remove }) => (
                <>
                  {value.map(({ key, name, ...restField }, index) => (
                    <Space
                      key={key}
                      style={{
                        display: 'flex',
                        marginBottom: '5px',
                        justifyContent: 'center',
                        position: 'relative',
                      }}
                      align='baseline'
                    >
                      <Form.Item
                        {...restField}
                        name={[name, 'fromField']}
                        rules={[
                          {
                            required: true,
                            message: '来源字段必填',
                          }
                        ]}
                      >
                        <TreeSelect
                          showSearch
                          style={{ width: '180px' }}
                          dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                          placeholder="请选择字段名 "
                          allowClear
                          treeDefaultExpandAll
                          showCheckedStrategy
                          onFocus={fromFieldFocus}
                        >
                          {fromField.map(s => <TreeNode value={s} title={s} key={s} />)}
                        </TreeSelect>
                      </Form.Item>
                      <div className={styles.centerImg}></div>
                      <Form.Item
                        {...restField}
                        name={[name, 'toField']}
                        rules={[
                          {
                            required: true,
                            message: '目标字段必填',
                          },
                          {
                            validator: (rules, value, callback) => {
                              const { field } = form.getFieldsValue()
                              let toFields = Array.from(new Set([...field.map(v => v.toField)]))
                              if (toFields.length !== field.length) {
                                return Promise.reject('目标字段不能重复！')
                              }
                              return Promise.resolve()
                            }
                          }
                        ]}
                      >
                        <TreeSelect
                          showSearch
                          style={{ width: '180px' }}
                          dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                          placeholder="请选择字段名 "
                          allowClear
                          treeDefaultExpandAll
                          showCheckedStrategy
                          onFocus={toFieldFocus}
                        >
                          {toField.map(s => <TreeNode value={s} title={s} key={s} />)}
                        </TreeSelect>
                      </Form.Item>
                      <Form.Item
                        hidden={true}
                        {...restField}
                        name={[name, 'layerName']}
                      >
                        <Input />
                      </Form.Item>
                      <MinusCircleOutlined onClick={() => remove(name)} />
                    </Space>
                  ))}
                  <Form.Item style={{ padding: '0 10px' }}>
                    <Button type="dashed" onClick={() => add()} block icon={<PlusOutlined />}>
                      添加映射
                    </Button>
                  </Form.Item>
                </>
              )}
            </Form.List>
          </Space>
        </Form>
      </Drag>
    </div>
  )

}

export default OtherSource