import React, { useMemo } from 'react'
import { Select } from 'antd'

const options = [
  {
    name: 'email',
    label: '邮箱(email)',
    value: '^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(.[a-zA-Z0-9_-]+)+$',
    message: '邮箱校验未通过',
  },
  {
    name: 'identity',
    label: '身份证号(identity)',
    value: '(^\\d{15}$)|(^\\d{18}$)|(^\\d{17}(\\d|X|x)$)',
    message: '身份证号校验未通过',
  },
  {
    name: 'mobile',
    label: '手机号(mobile)',
    value: '^[1][3,4,5,6,7,8,9][0-9]{9}$',
    message: '手机号校验未通过',
  },
  {
    name: 'bankAccount',
    label: '银行卡号(bankAccount)',
    value: '^(\\d{16}|\\d{19})$',
    message: '银行卡号校验未通过',
  },
]

const VerifyTextInput = (props) => {

  const { value, onChange, disabled } = props

  const valueShow = useMemo(() => {
    return Array.isArray(value) && value.length ? value[0].pattern : null
  }, [value])

  const selectChange = (value) => {
    if (value) {
      const item = options.find(v => v.value === value)
      onChange([{ name: item.name, pattern: value, message: item.message }])
    } else {
      onChange([])
    }
  }

  return (
    <Select
      disabled={disabled}
      allowClear
      value={valueShow}
      options={options}
      onChange={selectChange}
    />
  )

}

export default VerifyTextInput