import React, { useState, useEffect, useRef, useImperativeHandle, forwardRef, useMemo, useLayoutEffect } from 'react'
import styles from './index.less'
import { request } from '@wisdom-utils/utils'
import { Editor, Toolbar } from '@wangeditor/editor-for-react'
import { DomEditor } from '@wangeditor/editor'
import '@wangeditor/editor/dist/css/style.css'

const uploadFileAPI = `/PandaWorkFlow/WorkFlow/AccountManage/UploaderFiles`;
const downFileAPI = `/PandaWorkFlow/WorkFlow/AccountManage/DownloadFiles`;

const RichText = forwardRef(({ schema, onChange, value }, ref) => {

  const { presetValue, disabled, placeholder } = schema

  useImperativeHandle(ref, () => ({
    setHtml,
    onClear: () => editor.onClear(),
    getHtml: () => editor.txt.html(),
    getText: () => editor.txt.text(),
  }))

  const [editor, setEditor] = useState(null)

  const valueShow = useMemo(() => {
    return value || presetValue
  }, [presetValue, value])

  const editorChange = (editor) => {
    onChange(editor.getHtml())
  }

  useEffect(() => {
    onChange(presetValue)
  }, [presetValue])

  useEffect(() => {
    return () => {
      if (editor == null) return
      editor.destroy()
      setEditor(null)
    }
  }, [editor])

  return (
    <div className={styles.richText} disabled={disabled}>
      {
        !disabled ?
          <Toolbar
            editor={editor}
            defaultConfig={{
              excludeKeys: ['insertLink', 'group-image', 'group-video', 'codeBlock', 'undo', 'redo', 'fullScreen']
            }}
            mode="default"
            style={{ borderBottom: '1px solid #ccc' }}
          /> : null
      }
      <Editor
        defaultConfig={{
          readOnly: disabled,
          placeholder: disabled ? (placeholder || '') : (placeholder || '请输入内容'),
          focus: false,
          pasteFilterStyle: true, // 忽略粘贴样式
          pasteIgnoreImg: true, // 忽略粘贴的图片
          styleWithCSS: false,
          zIndex: 500,
          // customUploadImg: async (resultFiles, insertImgFn) => {
          //   console.log('resultFiles', resultFiles)
          //   // const result = await request({ url: uploadFileAPI, method: 'POST', data: formData })
          // }
          MENU_CONF: {
            uploadImage: {
              customInsert(res, inserFn) {
                const formData = new FormData();
                formData.append('file', file);
              },
              // onBeforeUpload: (e) => {
              //   const file = e?.clipboardData?.items[0]?.getAsFile() || null;
              //   if (!file) return;
              //   const formData = new FormData();
              //   formData.append('file', file);
              //   request({
              //     url: uploadFileAPI,
              //     method: 'POST',
              //     data: formData,
              //   }).then(res => {
              //     console.log(res)
              //   })
              // },
              server: uploadFileAPI,
            }
          }
        }}
        value={valueShow}
        onCreated={setEditor}
        onChange={editorChange}
        mode="default"
        style={{ minHeight: '300px', overflowY: 'hidden' }}
      />
    </div>
  )

})

export default RichText;
